package OsloAdressCoordinatesFinder;

import com.google.gson.*;
import com.google.gson.reflect.TypeToken;
import org.apache.commons.io.IOUtils;
import org.jsoup.Jsoup;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

public class App
{

    // Recommended using several VPN connections, Geocode API request limit per 24 hours is 2500

    private static final String geoCodingUrl = "http://maps.googleapis.com/maps/api/geocode/json?address=";
    private static final String adressFileName = "osloAdressesAndCoordinates.json";
    private static final String adressOutputFileName = "osloAdressesAndCoordinates.json";

    private static List<Adress> adresses = new ArrayList<>();

    public static void main( String[] args ) throws IOException {
        loadAdresses();
        getCoordinates();
        saveAdresses();
    }


    private static void loadAdresses() {

        try(InputStream reader = new FileInputStream(adressFileName)) {

            String json = IOUtils.toString(reader, Charset.forName("utf-8"));
            Gson gson = new GsonBuilder().create();
            adresses = gson.fromJson(json, new TypeToken<List<Adress>>(){}.getType());

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void saveAdresses() throws IOException {

        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        String json = gson.toJson(adresses);

        try(BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(adressOutputFileName), Charset.forName("utf-8")))) {
            writer.write(json);
            System.out.println("Write file success!");
        }
    }


    private static void getCoordinates() throws IOException {

        for(Adress adress : adresses) {

            // If the adress has been read successfully earlier, continue to next address
            if(adress.lat != null && adress.lng != null) continue;

            String number = correctNumbers(adress.numbers);
            String bydel = "OSLO";

            String url = geoCodingUrl + URLEncoder.encode(adress.name + ", " + adress.numbers + ", " + bydel, "utf-8");

            System.out.println(url);

            try {

                // Requests cannot exceed 10 requests per second, sleep 150ms
                Thread.sleep(150);

                // Fetch json from geocode api
                String json = Jsoup.connect(url).ignoreContentType(true).execute().body();

                // Parse the json returned from geocode api
                JsonParser parser = new JsonParser();

                JsonElement element = parser.parse(json);
                JsonObject jsonObject = element.getAsJsonObject();

                // Find correct element for location
                JsonObject locationJsonObject = jsonObject.get("results")
                        .getAsJsonArray()
                        .get(0)
                        .getAsJsonObject()
                        .get("geometry")
                        .getAsJsonObject()
                        .get("location")
                        .getAsJsonObject();

                String lat = locationJsonObject.get("lat").getAsString();
                String lng = locationJsonObject.get("lng").getAsString();

                adress.lat = lat;
                adress.lng = lng;

                System.out.println(lat + ", " + lng);
                adress.error = false;
            }
            catch(Exception e) {
                adress.error = true;
            }
        }
    }


    private static String correctNumbers(String numbers) {

        int index = numbers.indexOf(',');

        if(index > 0)
            return numbers.substring(0, index);

        return numbers;
    }
}
