package OsloAdressCrawler;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.*;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

public class App
{

    private static final String root = "http://www.oslo.kommune.no/om_oslo_kommune/bydelsoversikt/";
    private static List<Adress> osloAdresses = new ArrayList<>();

    public static void main( String[] args ) throws IOException {
        parseRoot();


        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        String json = gson.toJson(osloAdresses);

        System.out.println("Found " + osloAdresses.size() + " adresses!");
        System.out.println("Writing to file.");

        try(BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("osloAdresses.json"), Charset.forName("utf-8")))) {
            writer.write(json);
            System.out.println("Write file success!");
        }
    }


    private static void parseRoot() throws IOException {

        Document rootDocument = Jsoup.connect(root).get();
        List<String> subPages = new ArrayList<>();


        Elements alphabet = rootDocument.select("div[class=alfabet] > ul > li");

        for(Element letter : alphabet) {
            String url = letter.select("a").attr("href");
            subPages.add(url);
        }

        System.out.println("Found " + subPages.size() + " subpages.");

        parseSubPages(subPages);
    }


    private static void parseSubPages(List<String> subPages) throws IOException {

        for(String subPage : subPages) {
            System.out.println("Parsing " + subPage);
            Document subDocument = Jsoup.connect(subPage).get();

            Elements adresses = subDocument.select("table > tbody > tr");

            for(Element adress : adresses) {
                Elements parts = adress.select("td");

                String name = parts.get(0).text().trim();
                String number = parts.get(1).text().trim();
                String bydel = parts.get(2).text().trim();

                System.out.println("Found adress: " + name + ", " + number + ", " + bydel);

                osloAdresses.add(new Adress(name, number, bydel));
            }
        }
    }
}
