package no.oslotuc.apps.logviewer;

import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonReader;
import fi.iki.elonen.NanoHTTPD;
import fi.iki.elonen.NanoHTTPD.Response;
import no.oslotuc.models.QuestionLog;
import no.oslotuc.models.TUCQuery;
import no.oslotuc.net.mimetypes.MIME_TYPE;
import no.oslotuc.settings.Settings;
import no.oslotuc.util.ErrorPages;
import no.oslotuc.util.SessionReader;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.lang.reflect.ParameterizedType;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

public class LogController {

    private static Logger logger = LogManager.getLogger(LogController.class.getName());

    public static Response getLogViewer(NanoHTTPD.IHTTPSession session) {
        try {
            return new Response(Response.Status.OK, NanoHTTPD.MIME_HTML, LogViews.getLogView());

        } catch (FileNotFoundException e) {
            logger.error(e);
            return new Response(Response.Status.NOT_FOUND, NanoHTTPD.MIME_HTML, ErrorPages.get404NotFoundPage());
        }
    }


    public static Response getLogs(NanoHTTPD.IHTTPSession session) {

        List<QuestionLog> questionLogs = new ArrayList<QuestionLog>();

        File settingsDirectory = Paths.get(Settings.logDirectoryPath).toFile();

        if(!settingsDirectory.exists())
            return new Response(Response.Status.NOT_FOUND, MIME_TYPE.APPLICATION_JSON.toString(), "No logs found");

        File[] files = settingsDirectory.listFiles();

        if(files.length == 0)
            return new Response(Response.Status.NOT_FOUND, MIME_TYPE.APPLICATION_JSON.toString(), "No logs found");

        boolean error = false;

        for(File file : files) {

            try(InputStream fileInputStream = new FileInputStream(file)) {
                questionLogs.add(new GsonBuilder().create().fromJson(IOUtils.toString(fileInputStream, "utf-8"), QuestionLog.class));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                error = true;
            } catch (IOException e) {
                e.printStackTrace();
                error = true;
            }
        }

        if(error)
            return new Response(Response.Status.INTERNAL_ERROR, MIME_TYPE.APPLICATION_JSON.toString(), "500 - Internal Server Error");

        return new Response(Response.Status.OK, MIME_TYPE.APPLICATION_JSON.toString(), new GsonBuilder().create().toJson(questionLogs));
    }
}
