package no.oslotuc.apps.search;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import fi.iki.elonen.NanoHTTPD;
import fi.iki.elonen.NanoHTTPD.Response;
import no.oslotuc.models.TUCAnswer;
import no.oslotuc.models.TUCQuery;
import no.oslotuc.net.mimetypes.MIME_TYPE;
import no.oslotuc.prolog.TucToSicstus;
import no.oslotuc.settings.Settings;
import no.oslotuc.util.SessionReader;
import se.sics.jasper.Jasper;
import se.sics.jasper.Prolog;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

public class SearchController {


    public static Response search(NanoHTTPD.IHTTPSession session) {

        String postData = SessionReader.getAjaxPostData(session);

        if(postData.isEmpty())
            return new Response(Response.Status.BAD_REQUEST, MIME_TYPE.APPLICATION_JSON.toString(), "{\"message\":\"Post data was empty\"");

        TUCQuery query = new GsonBuilder().create().fromJson(postData, TUCQuery.class);
        String answer = performSearch(query.query);
        return new Response(Response.Status.OK, MIME_TYPE.APPLICATION_JSON.toString(), answer);
    }


    private static synchronized String performSearch(String query) {

        TucToSicstus tucToSicstus = new TucToSicstus();
        TUCAnswer answer = tucToSicstus.queryOracle(query);

        if(answer == null) {
            return "En feil skjedde.";
        }

        answer = answer.formatToWeb();

        Gson gson = new GsonBuilder().create();
        return gson.toJson(answer);
    }
}
