package no.oslotuc.net.urls;

import fi.iki.elonen.NanoHTTPD;
import fi.iki.elonen.NanoHTTPD.IHTTPSession;
import fi.iki.elonen.NanoHTTPD.Response;
import no.oslotuc.util.ErrorPages;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;

public class UrlManager {


    private static Logger logger = LogManager.getLogger(UrlManager.class.getName());
    private static UrlManager urlManager;
    private UrlManager() { }


    public static UrlManager getInstance() {
        if(urlManager == null)
            urlManager = new UrlManager();

        return urlManager;
    }


    public NanoHTTPD.Response getResponse(IHTTPSession session) {

        String url = session.getUri();
        boolean error = false;


        for(UrlPattern urlPattern : Urls.URLS) {

            if(urlPattern.getUrlPattern().matcher(url).matches()) {

                try {

                    List<Object> arguments = new ArrayList<Object>();
                    arguments.add(session);
                    Class<?>[] classArguments = new Class<?>[1];

                    Matcher matcher = urlPattern.getUrlPattern().matcher(url);

                    if(matcher.find()) {

                        int groupCount = matcher.groupCount();

                        if(groupCount > 0) {
                            classArguments = new Class<?>[groupCount+1];
                            for(int i = 0; i < matcher.groupCount(); i++) {
                                arguments.add(matcher.group(i+1));
                                classArguments[i+1] = String.class;
                            }
                        }
                    }

                    classArguments[0] = IHTTPSession.class;

                    Method method = urlPattern.getCls().getMethod(urlPattern.getMethodName(), classArguments);

                    if(arguments.size() == 0)
                        return (NanoHTTPD.Response) method.invoke(null, session);
                    else
                        return (NanoHTTPD.Response) method.invoke(null, arguments.toArray());

                } catch (NoSuchMethodException e) {
                    error = true;
                    logger.error(e);
                } catch (InvocationTargetException e) {
                    error = true;
                    logger.error(e);
                } catch (IllegalAccessException e) {
                    error = true;
                    logger.error(e);
                } catch (Exception e) {
                    error = true;
                    logger.error(e);
                }

                break;
            }
        }

        if(error) {
            return new Response(Response.Status.INTERNAL_ERROR, NanoHTTPD.MIME_HTML, "500 Internal Server Error");
        }

        return new Response(Response.Status.NOT_FOUND, NanoHTTPD.MIME_HTML, ErrorPages.get404NotFoundPage());
    }
}
