package no.oslotuc.prolog;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonSyntaxException;
import no.oslotuc.models.QuestionLog;
import no.oslotuc.models.TUCAnswer;
import no.oslotuc.settings.Settings;
import no.oslotuc.util.FileDeleter;
import no.oslotuc.util.QuestionWriter;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import se.sics.jasper.Query;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TucToSicstus {

    private Query query;
    private Map queryResult = new HashMap();
    private Logger logger = LogManager.getLogger(TucToSicstus.class.getName());


    public TUCAnswer queryOracle(String question) {

        QuestionLog questionLog = new QuestionLog();
        questionLog.startTime = System.currentTimeMillis();
        questionLog.question = question;

        String fileName = nanoTimeToStringRepresentation(System.nanoTime());
        String queryString = String.format("jettyrun(\"%s %s\").", fileName, question);
        String answerCorrectJson = "";

        try {
            query = Settings.prolog.openPrologQuery(queryString, queryResult);
            while(query.nextSolution()) {}

            String answer = readFileAsString(fileName);
            answerCorrectJson = answer.replaceFirst("\\}\\n\\{\\n\"route\"", ",\"route\"");
            Gson gson = new GsonBuilder().setPrettyPrinting().create();
            TUCAnswer tucAnswer = gson.fromJson(answerCorrectJson, TUCAnswer.class);

            //logger.info("User asked question: '" + question + "', and got answer:\n\n" + answer + "\n\n");
            questionLog.answer = tucAnswer;
            questionLog.endTime = System.currentTimeMillis();

            return tucAnswer;

        } catch (JsonSyntaxException e) {

            TUCAnswer tucAnswer = new TUCAnswer();
            tucAnswer.busstuc = answerCorrectJson;

            questionLog.answer = tucAnswer;
            questionLog.endTime = System.currentTimeMillis();

            return tucAnswer;

        } catch (NoSuchMethodException e) {
            logger.error(e);
        } catch (InterruptedException e) {
            logger.error(e);
        } catch (Exception e) {
            logger.error(e);
        } finally {
            File file = new File(fileName);
            if(!file.delete()) {
                logger.warn("File '" + fileName + "' could not be deleted. Starting FileDeleter!");
                new FileDeleter(fileName).start();
            }

            new QuestionWriter(questionLog).saveAnswer();
        }

        return null;
    }


    /*
        Jettyrun can't handle numbers as filename
     */
    private String nanoTimeToStringRepresentation(long nanoTime) {
        String time = "";

        for(char c : String.valueOf(nanoTime).toCharArray())
            time += (char)(97 + Integer.parseInt(String.valueOf(c)));

        return time;
    }


    private String readFileAsString(String osloTucAnswerFileName) {

        Path osloTucAnswerPath = Paths.get(osloTucAnswerFileName);

        if(!osloTucAnswerPath.toFile().exists())
            return null;


        try {
            List<String> answerLines = Files.readAllLines(osloTucAnswerPath, Charset.forName("utf-8"));
            StringBuilder builder = new StringBuilder();
            boolean jsonDetected = false;

            for(String line : answerLines) {

                if(line.replaceAll("^~+$", "~").equals("~")) {
                    jsonDetected = !jsonDetected;
                    continue;
                }

                if(!jsonDetected) continue;
                builder.append(line + "\n");
            }

            //No Json was found at all, fake json
            if(builder.toString().isEmpty()) {

                builder.append("{ \"busstuc\":");

                boolean dotsDetected = false;

                for(String line : answerLines) {

                    if(!dotsDetected) {
                        if(line.replaceAll("^\\.+$", "\\.").equals(".")) {
                            dotsDetected = true;
                            continue;
                        }
                    }

                    if(!line.trim().isEmpty())
                        builder.append("\"" + line + "\"");
                }

                builder.append("}");
            }

            return builder.toString();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return null;
    }
}
