package no.oslotuc.util;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

public class FileDeleter implements Runnable {


    private String filePath;
    private Logger logger = LogManager.getLogger(FileDeleter.class.getName());

    public FileDeleter(String filePath) {
        this.filePath = filePath;
    }

    @Override
    public void run() {

        boolean error = false;

        try {
            // Wait 10 seconds before deleting file
            logger.info("Trying to delete file '" + filePath + "' after 10 seconds.");
            Thread.sleep(10000);

            Path path = Paths.get(filePath);

            if(!path.toFile().exists()) return;
            Files.delete(Paths.get(filePath));

        } catch (IOException e) {
            logger.error(e);
            error = true;
        } catch (InterruptedException e) {
            logger.error(e);
            error = true;
        }

        if(error) {
            logger.warn("Could not delete '" + filePath + "'. Attempting deletion On Exit.");
            File file = new File(filePath);
            file.deleteOnExit();
        }
    }

    public void start() {
        new Thread(this).start();
    }
}
