package PrologConverter;

import RegToppParser.control.FileTaskDelegator;
import RegToppParser.interfaces.ParseListener;
import RegToppParser.models.Field;
import RegToppParser.models.FileType;
import RegToppParser.models.Post;
import RegToppParser.models.Version;
import RegToppParser.models.regtopp.TIX;

import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Created on 17.11.14.
 * @author Espen Jacobsson
 */
public class RegBus extends RegWriter implements ParseListener {


    private Set<String> buses = new LinkedHashSet<>();
    private Set<String> routes = new LinkedHashSet<>();


    public RegBus() {
        super("regbus.pl");
    }


    public void ready(Version version) {

        TIX tix = (TIX) FileTaskDelegator.getInstance().getFileParser(FileType.TIX);

        int lineNumber = 0;
        int tourNumber = 0;


        for(Post post : tix.getPosts()) {

            switch (version) {
                case V_1_1_D: {
                    lineNumber = post.getFieldValue(Field.TIX_V_1_1_D_LINJENR);
                    tourNumber = post.getFieldValue(Field.TIX_V_1_1_D_TURNR);
                    break;
                }

                case V_1_2: {
                    lineNumber = post.getFieldValue(Field.TIX_V_1_2_LINJENR);
                    tourNumber = post.getFieldValue(Field.TIX_V_1_2_TURNR);
                    break;
                }

                case V_1_3_A: {
                    lineNumber = post.getFieldValue(Field.TIX_V_1_3_A_LINE_NUMBER);
                    tourNumber = post.getFieldValue(Field.TIX_V_1_3_A_TRIP_NUMBER);
                    break;
                }
            }

            String lineNumberStripped = String.valueOf(lineNumber);

            String busProlog = String.format("regbus('%s').", lineNumberStripped);
            String routeProlog = String.format("route(bus_%d_%d,'%s','%s').", lineNumber, tourNumber, lineNumberStripped, lineNumberStripped);

            buses.add(busProlog);
            routes.add(routeProlog);
        }

        writeHeader();
        writeProlog();
        saveAndCloseFile();
    }


    private void writeHeader() {
        writeLine("/* -*- Mode:Prolog; coding:utf-8; -*- */");
        writeLine("");
    }


    private void writeProlog() {

        for(String bus : buses) {
            writeLine(bus);
        }

        writeLine("");

        for(String route : routes) {
            writeLine(route);
        }
    }
}
