package PrologConverter;

import RegToppParser.control.FileTaskDelegator;
import RegToppParser.interfaces.ParseListener;
import RegToppParser.models.Field;
import RegToppParser.models.FileType;
import RegToppParser.models.Post;
import RegToppParser.models.Version;
import RegToppParser.models.regtopp.DKO;

/**
 * Created on 17.11.14.
 * @author Espen Jacobsson
 */
public class RegDko extends RegWriter implements ParseListener {


    public RegDko() {
        super("regdko.pl");
    }


    @Override
    public void ready(Version version) {

        DKO dko = (DKO) FileTaskDelegator.getInstance().getFileParser(FileType.DKO);

        int date = dko.getDate();
        int weekdayForDate = dko.getWeekdayForDate();

        writeHeader();
        writeDateInfo(date, weekdayForDate);

        for(Post post : dko.getPosts()) {

            int administrationCode = 0;
            int indexNumber = 0;
            int dayCodeNumber = 0;
            String dayCode = "";

            switch (version) {
                case V_1_1_D: {
                    administrationCode = post.getFieldValue(Field.DKO_V_1_1_D_ADMINISTRASJONSKODE);
                    indexNumber = post.getFieldValue(Field.DKO_V_1_1_D_LOPENR);
                    dayCodeNumber = post.getFieldValue(Field.DKO_V_1_1_D_DAGKODENR);
                    dayCode = post.getFieldValue(Field.DKO_V_1_1_D_DAGKODE);
                    break;
                }
                case V_1_2: {
                    administrationCode = post.getFieldValue(Field.DKO_V_1_2_ADMINISTRASJONSKODE);
                    indexNumber = post.getFieldValue(Field.DKO_V_1_2_LOPENR);
                    dayCodeNumber = post.getFieldValue(Field.DKO_V_1_2_DAGKODENR);
                    dayCode = post.getFieldValue(Field.DKO_V_1_2_DAGKODE);
                    break;
                }
                case V_1_3_A: {
                    administrationCode = post.getFieldValue(Field.DKO_V_1_3_A_ADMINISTRATION_CODE);
                    indexNumber = post.getFieldValue(Field.DKO_V_1_3_A_INDEX_NUMBER);
                    dayCodeNumber = post.getFieldValue(Field.DKO_V_1_3_A_DAY_CODE_NUMBER);
                    dayCode = post.getFieldValue(Field.DKO_V_1_3_A_DAY_CODE);
                    break;
                }
            }

            String firstWeekInPeriod = dayCode.substring(0, 7);
            String secondWeekInPeriod = dayCode.substring(7, 14);
            String thirdWeekInPeriod = dayCode.substring(14, 21);
            String fourthWeekInPeriod = dayCode.substring(21, 28);


            String line = String.format("dko(%d,%d,%d,%s,%s,%s,%s,'%s').",
                                        dayCodeNumber,
                                        administrationCode,
                                        indexNumber,
                                        firstWeekInPeriod,
                                        secondWeekInPeriod,
                                        thirdWeekInPeriod,
                                        fourthWeekInPeriod,
                                        dayCode);

            writeLine(line);
        }

        saveAndCloseFile();
    }


    private void writeHeader() {
        writeLine("/* -*- Mode:Prolog; coding:utf-8; -*- */");
    }


    private void writeDateInfo(int date, int weekdayForDate) {
        writeLine(String.format("dkodate(%d,%d).", date, weekdayForDate));
        writeLine("");
    }
}
