package PrologConverter;

import RegToppParser.control.FileTaskDelegator;
import RegToppParser.interfaces.ParseListener;
import RegToppParser.models.Field;
import RegToppParser.models.FileType;
import RegToppParser.models.Post;
import RegToppParser.models.Version;
import RegToppParser.models.regtopp.HPL;
import java.text.Normalizer;
import java.util.List;
import java.util.regex.Pattern;

/**
 * Created on 17.11.14.
 * @author Espen Jacobsson
 */
public class RegHpl extends RegWriter implements ParseListener {


    public RegHpl() {
        super("reghpl.pl");
    }


    @Override
    public void ready(Version version) {

        writeHeader();

        HPL hpl = (HPL) FileTaskDelegator.getInstance().getFileParser(FileType.HPL);
        List<Post> hplPosts = hpl.getPosts();

        for(Post post : hplPosts) {

            int stationId = 0;
            String stationName = "";

            switch(version) {
                case V_1_1_D: {
                    stationId = post.getFieldValue(Field.HPL_V_1_1_D_HOLDEPLASSNR);
                    stationName = post.<String>getFieldValue(Field.HPL_V_1_1_D_FULLSTENDIG_NAVN).trim();
                    break;
                }

                case V_1_2: {
                    stationId = post.getFieldValue(Field.HPL_V_1_2_HOLDEPLASSNR);
                    stationName = post.<String>getFieldValue(Field.HPL_V_1_2_FULLSTENDIG_NAVN).trim();
                    break;
                }

                case V_1_3_A: {
                    stationId = post.getFieldValue(Field.HPL_V_1_3_A_STOP_NUMBER);
                    stationName = post.<String>getFieldValue(Field.HPL_V_1_3_A_COMPLETE_NAMES).trim();
                    break;
                }
            }

            writeStops(stationId, stationName);
        }
        saveAndCloseFile();
    }


    private void writeHeader() {
        writeLine("/* -*- Mode:Prolog; coding:utf-8; -*- */");
    }

    private void writeStops(int stationId, String stationName) {
        String cleanedStationName = processForAtom(deAccent(stationName));
        writeLine(String.format("hpl('%d',%s,%s,'%s').", stationId, cleanedStationName, cleanedStationName, stationName));
    }


    private String deAccent(String str) {

        str = str.replaceAll("å", "#");
        str = str.replaceAll("Å", "@");

        String nfdNormalizedString = Normalizer.normalize(str, Normalizer.Form.NFC);
        Pattern pattern = Pattern.compile("\\p{InCombiningDiacriticalMarks}+");
        str = pattern.matcher(nfdNormalizedString).replaceAll("");
        str = str.replaceAll("#", "å");
        str = str.replaceAll("@", "Å");
        return str;
    }

    private String processForAtom(String str) {
        str = str.toLowerCase();
        str = str.replaceAll("\\s+", "_");
        return str;
    }
}
