package PrologConverter.regpas;

import PrologConverter.RegWriter;
import RegToppParser.interfaces.ParseListener;
import RegToppParser.models.*;

import java.util.*;

/**
 * Created on 17.11.14.
 * @author Espen Jacobsson
 */
public class RegPasBase extends RegWriter implements ParseListener {


    protected Set<Tour> tours = new LinkedHashSet<>();

    public RegPasBase() {
        super("regpas.pl");
    }


    public void ready(Version version) {

        switch (version) {
            case V_1_1_D: {
                new RegPasVersion1();
                break;
            }
            case V_1_2: {
                new RegPasVersion2();
                break;
            }
        }
    }


    protected void writeHeader() {
        writeLine("/* -*- Mode:Prolog; coding:utf-8; -*- */");
        writeLine(  "%% Generated by RegToppParser.PrologConverter.regpas.RegPasBase.java on " + new Date().toGMTString() + "\n\n" +
                    "%% PASSES4 (TOURTYPE, STATION_NR, STATION, SEQ_NR, ARR_DELAY, DEP_DELAY)\n" +
                    "%%    Example: passes4(17326,16011035,anton_grevskotts_vei,22,15,16).\n\n" +
                    "%% NTOURSTOPS (TOURTYPE,LENGTH)\n" +
                    "%%    Defines the number of stations in a tour, e.g. ntourstops(12713,5).");
        writeLine("");
    }



    protected void writeProlog() {

        writeLine("%% PASSES4 (TOURTYPE, STATION NUMBER, STATION, SEQNUMBER, ARR DELAY, DEP DELAY)");

        for(Tour tour : tours) {
            for(TourItem tourItem : tour.getTourItems()) {
                String line = String.format("passes4(%s,%s,%s,%d,%d,%d).",
                        tour.getPostIndexInTDA(),
                        tourItem.getStopNumber(),
                        tourItem.getStopName(),
                        tourItem.getSequenceNumber(),
                        tourItem.getDrivingTimeArrival(),
                        tourItem.getDrivingTimeDeparture());

                writeLine(line);
            }
        }

        writeLine("");

        for(Tour tour : tours) {
            String line = String.format("ntourstops(%s,%s).", tour.getPostIndexInTDA(), tour.getNumberOfStops());
            writeLine(line);
        }
    }
}
