package PrologConverter.regpas;

import RegToppParser.control.FileTaskDelegator;
import RegToppParser.models.Field;
import RegToppParser.models.FieldValuePair;
import RegToppParser.models.FileType;
import RegToppParser.models.Post;
import RegToppParser.models.regtopp.HPL;
import RegToppParser.models.regtopp.TDA;
import RegToppParser.models.regtopp.TIX;
import RegToppParser.util.FieldValueUtil;

import java.util.List;

/**
 * Created on 23.12.14.
 * @author Espen
 */
public class RegPasVersion1 extends RegPasBase {


    public RegPasVersion1() {
        writeHeader();
        createTours();
        writeProlog();
        saveAndCloseFile();
    }


    private void createTours() {

        TIX tix = (TIX) FileTaskDelegator.getInstance().getFileParser(FileType.TIX);
        TDA tda = (TDA) FileTaskDelegator.getInstance().getFileParser(FileType.TDA);
        HPL hpl = (HPL) FileTaskDelegator.getInstance().getFileParser(FileType.HPL);

        // Check all posts in the TIX file
        for(final Post tixPost : tix.getPosts()) {

            int postNumberInTDA = tixPost.getFieldValue(Field.TIX_V_1_1_D_POSTNR_I_TURDATA);
            int numberOfStops = tixPost.getFieldValue(Field.TIX_V_1_1_D_ANTALL_HOLDEPLASSER);

            // Create a new tour so we later can check if the tour is a duplicate
            Tour tour = new Tour(String.valueOf(postNumberInTDA), String.valueOf(numberOfStops));

            int startIndex = postNumberInTDA - 1;
            int endIndex = startIndex + numberOfStops;

            // The TIX file gives us the line number in the TDA file from which point the tour is listed
            for(int lineNumberInTDA = startIndex; lineNumberInTDA < endIndex; lineNumberInTDA++) {

                Post tdaPost = tda.getPosts().get(lineNumberInTDA);
                String stopNumber = tdaPost.getFieldValue(Field.TDA_V_1_1_D_HOLDEPLASSNR);

                int sequenceNumber = lineNumberInTDA - startIndex + 1;
                int drivingTimeArrival = tdaPost.getFieldValue(Field.TDA_V_1_1_D_ANKOMSTTID);
                int drivingTimeDeparture = tdaPost.getFieldValue(Field.TDA_V_1_1_D_AVGANGSTID);

                // Get post where the primary key stopNumber is our stopNumber
                List<FieldValuePair> stopPrimaryKeys = getStopPrimaryKeys(tixPost, tdaPost);

                Post hplPost = hpl.getPost(stopPrimaryKeys);
                String stopName = hplPost.getFieldValue(Field.HPL_V_1_1_D_FULLSTENDIG_NAVN);

                // Create a tour item for this TDA post
                tour.addTourItem(stopNumber, stopName, sequenceNumber, drivingTimeArrival, drivingTimeDeparture);
            }

            tours.add(tour);
        }
    }

    private List<FieldValuePair> getStopPrimaryKeys(Post tixPost, Post tdaPost) {

        int administrationCode  = tixPost.getFieldValue(Field.TIX_V_1_1_D_ADMINISTRASJONSKODE);
        int indexNumber         = tixPost.getFieldValue(Field.TIX_V_1_1_D_LOPENR);
        int stopNumber          = tdaPost.getFieldValue(Field.TDA_V_1_1_D_HOLDEPLASSNR);

        return new FieldValueUtil()
                            .add(Field.HPL_V_1_1_D_ADMINISTRASJONSKODE, administrationCode)
                            .add(Field.HPL_V_1_1_D_LOPENUMMER, indexNumber)
                            .add(Field.HPL_V_1_1_D_HOLDEPLASSNR, stopNumber)
                            .get();
    }
}
