package PrologConverter.regpas;

import RegToppParser.control.FileTaskDelegator;
import RegToppParser.models.Field;
import RegToppParser.models.FieldValuePair;
import RegToppParser.models.FileType;
import RegToppParser.models.Post;
import RegToppParser.models.regtopp.HPL;
import RegToppParser.models.regtopp.TIX;
import RegToppParser.models.regtopp.TMS;
import RegToppParser.util.FieldValueUtil;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created on 23.12.14.
 * @author Espen
 */
public class RegPasVersion2 extends RegPasBase {


    public RegPasVersion2() {
        writeHeader();
        createTours();
        writeProlog();
        saveAndCloseFile();
    }


    private void createTours() {
        TIX tix = (TIX) FileTaskDelegator.getInstance().getFileParser(FileType.TIX);
        TMS tms = (TMS) FileTaskDelegator.getInstance().getFileParser(FileType.TMS);
        HPL hpl = (HPL) FileTaskDelegator.getInstance().getFileParser(FileType.HPL);

        Set<Post> visitedPosts = new HashSet<>();

        for(Post tixPost : tix.getPosts()) {

            List<FieldValuePair> primaryKeyValuePairs = getPrimaryKeys(tixPost);
            Post tmsPost = tms.getPost(primaryKeyValuePairs);

            while(tmsPost != null) {

                if(visitedPosts.contains(tmsPost)) break;
                visitedPosts.add(tmsPost);

                // Switch the sequence number to the next sequence number
                primaryKeyValuePairs.remove(primaryKeyValuePairs.size() - 1);
                primaryKeyValuePairs.add(new FieldValuePair(Field.TMS_V_1_2_SEKVENSNR_HPL, tmsPost.<Integer>getFieldValue(Field.TMS_V_1_2_SEKVENSNR_HPL) + 1));

                tmsPost = tms.getPost(primaryKeyValuePairs);
            }
        }
    }


    private List<FieldValuePair> getPrimaryKeys(Post tixPost) {
        return new FieldValueUtil()
                        .add(Field.TMS_V_1_2_ADMINISTRASJONSKODE, tixPost.getFieldValue(Field.TIX_V_1_2_ADMINISTRASJONSKODE))
                        .add(Field.TMS_V_1_2_LOPENR, tixPost.getFieldValue(Field.TIX_V_1_2_LOPENR))
                        .add(Field.TMS_V_1_2_LINJENR, tixPost.getFieldValue(Field.TIX_V_1_2_LINJENR))
                        .add(Field.TMS_V_1_2_RETNING, tixPost.getFieldValue(Field.TIX_V_1_2_RETNING))
                        .add(Field.TMS_V_1_2_TURMONSTERNR, tixPost.getFieldValue(Field.TIX_V_1_2_TURMONSTERNR))
                        .add(Field.TMS_V_1_2_SEKVENSNR_HPL, 1)
                        .get();
    }
}
