package RegToppParser.models;

import java.util.ArrayList;
import java.util.List;

/**
 * Created on 15.11.14.
 * @author Espen Jacobsson
 */
public enum Field {


    /******************************************************************
     *
     *   SYNTAX FOR FIELDS MUST BE: <FILETYPE>_<VERSION>_<FIELDNAME>
     *
     ******************************************************************/

    /*
        Turindeks (TURIX.TIX)
     */

    TIX_V_1_1_D_ADMINISTRASJONSKODE                             (true,  true,   1,  3, "Administrasjonskode", Integer.class),
    TIX_V_1_1_D_LOPENR                                          (true,  true,   4,  4, "Løpenr", Integer.class),
    TIX_V_1_1_D_LINJENR                                         (true,  true,   5,  8, "Linjenr", Integer.class),
    TIX_V_1_1_D_TURNR                                           (true,  true,   9, 12, "Turnr", Integer.class),
    TIX_V_1_1_D_TRAFIKKART                                      (false, false, 13, 15, "Trafikkart", Integer.class),
    TIX_V_1_1_D_DAGKODENUMMER                                   (false, true,  16, 19, "Dagkodenummer", Integer.class),          // Peker til post i DAGKODE
    TIX_V_1_1_D_SELSKAPSKODE                                    (false, false, 20, 22, "Selskapskode", Integer.class),
    TIX_V_1_1_D_MERKNAD_1                                       (false, false, 23, 25, "Merknad 1", Integer.class),              // Merknadsnummer
    TIX_V_1_1_D_MERKNAD_2                                       (false, false, 26, 28, "Merknad 2", Integer.class),              // Merknadsnummer
    TIX_V_1_1_D_DESTINASJONSNUMMER                              (false, false, 29, 32, "Destinasjonsnummer", Integer.class),
    TIX_V_1_1_D_PUBLIKUMSLINJENR                                (false, false, 33, 40, "Publikumslinjenr", String.class),
    TIX_V_1_1_D_RETNING                                         (false, true,  41, 41, "Retning", Integer.class),
    TIX_V_1_1_D_TURTYPE                                         (false, true,  42, 43, "Turtype", Integer.class),
    TIX_V_1_1_D_AVGANGSTID                                      (false, true,  44, 47, "Avgangstid", Integer.class),
    TIX_V_1_1_D_ANTALL_HOLDEPLASSER                             (false, false, 48, 50, "Antall holdeplasser", Integer.class),    // Antall poster i TURDATA
    TIX_V_1_1_D_POSTNR_I_TURDATA                                (false, false, 51, 57, "Postnr i TURDA", Integer.class),       // Peker til første holdeplass
    TIX_V_1_1_D_PAKKETRANSPORT                                  (false, false, 58, 58, "Pakketransport", Integer.class),
    TIX_V_1_1_D_TAKSTKODE                                       (false, true,  59, 59, "Takstkode", Integer.class),



    TIX_V_1_2_ADMINISTRASJONSKODE                               (true,  true,   1,  3, "Administrasjonskode", Integer.class),
    TIX_V_1_2_LOPENR                                            (true,  true,   4,  4, "Løpenr", Integer.class),
    TIX_V_1_2_LINJENR                                           (true,  true,   5,  8, "Linjenr", Integer.class),
    TIX_V_1_2_TURNR                                             (true,  true,   9, 12, "Turnr", Integer.class),
    TIX_V_1_2_TRANSPORTMIDDEL                                   (false, true,  13, 15, "Transportmiddel", Integer.class),
    TIX_V_1_2_DAGKODENR                                         (false, true,  16, 19, "Dagkodenr", Integer.class),                  // Peker til post i DAGKODE
    TIX_V_1_2_SELSKAPSKODE                                      (false, false, 20, 22, "Selskapskode", Integer.class),
    TIX_V_1_2_MERKNADSNR_1                                      (false, false, 23, 25, "Merknadsnr 1", Integer.class),               // Peker til post i MERKNAD
    TIX_V_1_2_MERKNADSNR_2                                      (false, false, 26, 28, "Merknadsnr 2", Integer.class),               // Peker til post i MERKNAD
    TIX_V_1_2_DESTINASJONSNR_AVGANG                             (false, false, 29, 32, "Destinasjonsnr (avgang)", Integer.class),    // Peker til post i DESTNAVN
    TIX_V_1_2_PUBLIKUMSLINJENR                                  (false, false, 33, 40, "Publikumslinjenr", String.class),
    TIX_V_1_2_RETNING                                           (false, true,  41, 41, "Retning", Integer.class),
    TIX_V_1_2_TURMONSTERNR                                      (false, true,  42, 43, "Turmønsternr", Integer.class),               // Peker til post i TURMSTR
    TIX_V_1_2_AVGANGSTID                                        (false, true,  44, 47, "Avgangstid", Integer.class),
    TIX_V_1_2_PAKKETRANSPORT                                    (false, false, 48, 48, "Pakketransport", Integer.class),
    TIX_V_1_2_TAKSTKODE                                         (false, false, 49, 49, "Takstkode", Integer.class),
    TIX_V_1_2_KUNNGJORINGSTYPE                                  (false, true,  50, 50, "Kunngjøringstype", Integer.class),
    TIX_V_1_2_TRAFIKKTYPE                                       (false, true,  51, 51, "Trafikktype", Integer.class),
    TIX_V_1_2_DESTINASJONSNR_ANKOMST                            (false, false, 52, 55, "Destinasjonsnr (ankomst)", Integer.class),    // Peker til post i DESTNAVN
    TIX_V_1_2_TRAFIKKERINGSDAGER                                (false, false, 56, 62, "Trafikkeringsdager", Integer.class),



    TIX_V_1_3_A_ADMINISTRATION_CODE                             (true,  true,   1,  3, "Administration code", Integer.class),
    TIX_V_1_3_A_INDEX_NUMBER                                    (true,  true,   4,  4, "Index number", Integer.class),
    TIX_V_1_3_A_LINE_NUMBER                                     (true,  true,   5,  8, "Line number", Integer.class),
    TIX_V_1_3_A_TRIP_NUMBER                                     (true,  true,   9, 12, "Trip number", Integer.class),
    TIX_V_1_3_A_TRANSPORT_MODE                                  (false, true,  13, 15, "Transport mode", Integer.class),
    TIX_V_1_3_A_DAY_CODE_NUMBER                                 (false, true,  16, 19, "Day code number", Integer.class),                            // Links to Day code
    TIX_V_1_3_A_COMPANY_CODE                                    (false, false, 20, 22, "Company code", String.class),
    TIX_V_1_3_A_REMARK_NUMBER_1                                 (false, false, 23, 25, "Remark number (1)", Integer.class),                          // Links to Remark
    TIX_V_1_3_A_REMARK_NUMBER_2                                 (false, false, 26, 28, "Remark number (2)", Integer.class),                          // Links to Remark
    TIX_V_1_3_A_DESTINATION_NUMBER_DEPARTURE                    (false, false, 29, 32, "Destination number (departure)", Integer.class),             // Links to Destination
    TIX_V_1_3_A_LINE_NUMBER_USED_IN_PASSENGER_INFORMATION       (false, true,  33, 40, "Line number used in passenger information", String.class),
    TIX_V_1_3_A_DIRECTION                                       (false, true,  41, 41, "Direction", Integer.class),
    TIX_V_1_3_A_TRIP_PATTERN_NUMBER                             (false, true,  42, 43, "Trip pattern number", Integer.class),                        // Links to Trip pattern
    TIX_V_1_3_A_TIME_OF_DEPARTURE                               (false, true,  44, 47, "Time of departure", Integer.class),
    TIX_V_1_3_A_DELIVERY_OF_PARCELS                             (false, false, 48, 48, "Delivery of parcels", Integer.class),
    TIX_V_1_3_A_FARE_CODE                                       (false, true,  49, 49, "Fare code", Integer.class),
    TIX_V_1_3_A_ANNOUNCEMENT_TYPE                               (false, true,  50, 50, "Announcement type", Integer.class),
    TIX_V_1_3_A_TRAFFIC_TYPE                                    (false, true,  51, 51, "Traffic type", Integer.class),
    TIX_V_1_3_A_DESTINATION_NUMBER_ARRIVAL                      (false, false, 52, 55, "Destination number (arrival)", Integer.class),               // Links to Destination
    TIX_V_1_3_A_DAYS_OF_TRAFFIC                                 (false, true,  56, 62, "Days of traffic", Integer.class),
    TIX_V_1_3_A_TRIP_ID                                         (false, true,  63, 71, "Trip-ID", Integer.class),                                   // Internal ID from plan system



    /*
        Stoppunkter (STP)
     */

    STP_V_1_3_A_ADMINISTRATION_CODE                             (true,  true,   1,  3, "Administration code", Integer.class),
    STP_V_1_3_A_INDEX_NUMBER                                    (true,  true,   4,  4, "Index number", Integer.class),
    STP_V_1_3_A_STOP_NUMBER                                     (true,  true,   5, 12, "Stop number", Integer.class),
    STP_V_1_3_A_STOP_POINT_NUMBER                               (true,  true,  13, 14, "Stop point number", Integer.class),
    STP_V_1_3_A_STOP_POINT_INDICATION                           (false, true,  15, 16, "Stop point indication", String.class),
    STP_V_1_3_A_X_COORDINATE                                    (false, true,  17, 26, "X-coordinate", Integer.class),
    STP_V_1_3_A_Y_COORDINATE                                    (false, true,  27, 36, "Y-coordinate", Integer.class),
    STP_V_1_3_A_PASSENGER_INFORMATION_DISPLAY                   (false, false, 37, 37, "Passenger information display", Integer.class),          // 0=no, 1=yes
    STP_V_1_3_A_DYNAMIC_PASSENGER_INFORMATION_DISPLAY           (false, false, 38, 38, "Dynamic passenger", Integer.class),                      // 0=no, 1=yes
    STP_V_1_3_A_BENCH                                           (false, false, 39, 39, "Bench", Integer.class),                                  // 0=no, 1=yes
    STP_V_1_3_A_SHELTER                                         (false, false, 40, 40, "Shelter", Integer.class),                                // 0=no, 1=yes
    STP_V_1_3_A_OPEN_FOR_A_DIFFERENT_INFRASTRUCTURE             (false, false, 41, 46, "Open for a different infrastructure", Integer.class),
    STP_V_1_3_A_STOP_POINT_DESCRIPTION                          (false, false, 47, 86, "Stop point description", String.class),
    STP_V_1_3_A_STOP_POINT_ID                                   (false, true,  87, 94, "Stop point-ID", String.class),                         //Internal ID from plan system



    /*
        Turmønster (TURMSTR.TMS)
     */

    TMS_V_1_2_ADMINISTRASJONSKODE                               (true,  true,   1,  3, "Administrasjonskode", Integer.class),
    TMS_V_1_2_LOPENR                                            (true,  true,   4,  4, "Løpenr", Integer.class),
    TMS_V_1_2_LINJENR                                           (true,  true,   5,  8, "Linjenr", Integer.class),
    TMS_V_1_2_RETNING                                           (true,  true,   9,  9, "Retning", Integer.class),
    TMS_V_1_2_TURMONSTERNR                                      (true,  true,  10, 11, "Turmønsternr", Integer.class),
    TMS_V_1_2_SEKVENSNR_HPL                                     (true,  true,  12, 14, "Sekvensnr hpl", Integer.class),
    TMS_V_1_2_HOLDEPLASSNR                                      (false, true,  15, 22, "Holdeplassnr", Integer.class),
    TMS_V_1_2_KJORETID_ANKOMST                                  (false, true,  23, 25, "Kjøretid ankomst", Integer.class),
    TMS_V_1_2_STOPPUNKTNR_ANKOMST                               (false, false, 26, 27, "Stoppunktnr ankomst", Integer.class),
    TMS_V_1_2_KJORETID_AVGANG                                   (false, true,  28, 30, "Kjøretid avgang", Integer.class),
    TMS_V_1_2_STOPPUNKTNR_AVGANG                                (false, false, 31, 32, "Stoppunktnr avgang", Integer.class),
    TMS_V_1_2_DESTINASJONSNR                                    (false, false, 33, 36, "Destinasjonsnr", Integer.class),         // Peker til post i DESTNAVN
    TMS_V_1_2_MERKNADSNR                                        (false, false, 37, 39, "Merknadsnr", Integer.class),             // Peker til post i MERKNAD
    TMS_V_1_2_MONITOR                                           (false, false, 40, 41, "Monitor", String.class),
    TMS_V_1_2_AVSTAND                                           (false, false, 42, 47, "Avstand", Integer.class),



    TMS_V_1_3_A_ADMINISTRATION_CODE                             (true,  true,   1,  3, "Administration code", Integer.class),
    TMS_V_1_3_A_INDEX_NUMBER                                    (true,  true,   4,  4, "Index number", Integer.class),
    TMS_V_1_3_A_LINE_NUMBER                                     (true,  true,   5,  8, "Line number", Integer.class),
    TMS_V_1_3_A_DIRECTION                                       (true,  true,   9,  9, "Direction", Integer.class),
    TMS_V_1_3_A_TRIP_PATTERN_NUMBER                             (true,  true,  10, 11, "Trip pattern number", Integer.class),
    TMS_V_1_3_A_SEQUENCE_NUMBER_FOR_STOPS                       (true,  true,  12, 14, "Sequence number for stop", Integer.class),
    TMS_V_1_3_A_STOP_TYPE                                       (false, true,  15, 15, "Stop type", Integer.class),                      // 0=Stop, 1=other
    TMS_V_1_3_A_STOP_NUMBER                                     (false, true,  16, 23, "Stop number", Integer.class),
    TMS_V_1_3_A_STOP_POINT_NUMBER_ARRIVAL                       (false, false, 24, 25, "Stop point number arrival", Integer.class),
    TMS_V_1_3_A_STOP_POINT_NUMBER_DEPARTURE                     (false, false, 26, 27, "Stop point number departure", Integer.class),
    TMS_V_1_3_A_DRIVING_TIME_ARRIVAL                            (false, true,  28, 31, "Driving time arrival", Integer.class),
    TMS_V_1_3_A_DRIVING_TIME_DEPARTURE                          (false, true,  32, 35, "Driving time departure", Integer.class),
    TMS_V_1_3_A_DESTINATION_NUMBER                              (false, true,  36, 39, "Destination number", Integer.class),             // Links to Destination
    TMS_V_1_3_A_REMARK_NUMBER                                   (false, false, 40, 42, "Remark number", Integer.class),                 // Links to Remark
    TMS_V_1_3_A_MONITOR                                         (false, false, 43, 44, "Monitor", String.class),
    TMS_V_1_3_A_DISTANCE                                        (false, true,  45, 50, "Distance", Integer.class),




    /*
        Turdata (TURDA.TDA)
     */

    TDA_V_1_1_D_HOLDEPLASSNR                                    (false, false,  1,  8, "Holdeplassnr", Integer.class),
    TDA_V_1_1_D_ANKOMSTTID                                      (false, false,  9, 11, "Ankomsttid", Integer.class),
    TDA_V_1_1_D_AVGANGSTID                                      (false, false, 12, 14, "Avgangstid", Integer.class),
    TDA_V_1_1_D_AVSTAND                                         (false, false, 15, 20, "Avstand", Integer.class),



    /*
        Holdeplass (HPL.HPL)
     */

    HPL_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,    1,  3, "Administrasjonskode", Integer.class),
    HPL_V_1_1_D_LOPENUMMER                                      (true, true,    4,  4, "Løpenummer", Integer.class),
    HPL_V_1_1_D_HOLDEPLASSNR                                    (true, true,    5, 12, "Holdeplassnr", Integer.class),
    HPL_V_1_1_D_FULLSTENDIG_NAVN                                (false, false, 13, 42, "Fullstendig navn", String.class),
    HPL_V_1_1_D_KORTNAVN                                        (false, false, 43, 47, "Kortnavn", String.class),
    HPL_V_1_1_D_SONE_KORTNAVN                                   (false, false, 48, 53, "Sone kortnavn", String.class),
    HPL_V_1_1_D_X_KOORDINAT                                     (false, false, 54, 63, "X-koordinat", Integer.class),
    HPL_V_1_1_D_Y_KOORDINAT                                     (false, false, 64, 73, "Y-koordinat", Integer.class),
    HPL_V_1_1_D_SONE_NR_1                                       (false, false, 74, 78, "Sonenr 1", Integer.class),
    HPL_V_1_1_D_SONE_NR_2                                       (false, false, 79, 83, "Sonenr 2", Integer.class),
    HPL_V_1_1_D_TYPE                                            (false, true,  84, 84, "Type", Integer.class),                   // Overgangsbeskrivelse
    HPL_V_1_1_D_SPES_OMSTIGNIGSTID                              (false, true,  85, 86, "Spes. omstigningstid", Integer.class),
    HPL_V_1_1_D_KLASSE                                          (false, true,  87, 87, "Klasse", Integer.class),



    HPL_V_1_2_ADMINISTRASJONSKODE                               (true, true,    1,  3, "Administrasjonskode", Integer.class),
    HPL_V_1_2_LOPENUMMER                                        (true, true,    4,  4, "Løpenummer", Integer.class),
    HPL_V_1_2_HOLDEPLASSNR                                      (true, true,    5, 12, "Holdeplassnr", Integer.class),
    HPL_V_1_2_FULLSTENDIG_NAVN                                  (false, true,  13, 42, "Fullstendig navn", String.class),
    HPL_V_1_2_KORTNAVN                                          (false, false, 43, 47, "Kortnavn", String.class),
    HPL_V_1_2_SONE_KORTNAVN                                     (false, false, 48, 53, "Sone kortnavn", String.class),
    HPL_V_1_2_X_KOORDINAT                                       (false, false, 54, 63, "X-koordinat", Integer.class),
    HPL_V_1_2_Y_KOORDINAT                                       (false, false, 64, 73, "Y-koordinat", Integer.class),
    HPL_V_1_2_SONE_NR_1                                         (false, false, 74, 78, "Sonenr 1", Integer.class),
    HPL_V_1_2_SONE_NR_2                                         (false, false, 79, 83, "Sonenr 2", Integer.class),
    HPL_V_1_2_TYPE                                              (false, true,  84, 84, "Type", Integer.class),                   // Overgangsbeskrivelse
    HPL_V_1_2_SPES_OMSTIGNIGSTID                                (false, true,  85, 86, "Spes. omstigningstid", Integer.class),
    HPL_V_1_2_KLASSE                                            (false, true,  87, 87, "Klasse", Integer.class),



    HPL_V_1_3_A_ADMINISTRATION_CODE                             (true, true,    1,  3, "Administration code", Integer.class),
    HPL_V_1_3_A_INDEX_NUMBER                                    (true, true,    4,  4, "Index number", Integer.class),
    HPL_V_1_3_A_STOP_NUMBER                                     (true, true,    5, 12, "Stop number", Integer.class),
    HPL_V_1_3_A_COMPLETE_NAMES                                  (false, true,  13, 42, "Complete names", String.class),
    HPL_V_1_3_A_SHORT_NAMES                                     (false, true,  43, 47, "Short names", String.class),
    HPL_V_1_3_A_ZONE                                            (false, true,  48, 53, "Zone", String.class),
    HPL_V_1_3_A_X_COORDINATE                                    (false, false, 54, 63, "X-coordinate", Integer.class),
    HPL_V_1_3_A_Y_COORDINATE                                    (false, false, 64, 73, "Y-coordinate", Integer.class),
    HPL_V_1_3_A_ZONE_NUMBER_1                                   (false, false, 74, 78, "Zone number 1", Integer.class),
    HPL_V_1_3_A_ZONE_NUMBER_2                                   (false, false, 79, 83, "Zone number 2", Integer.class),
    HPL_V_1_3_A_STOP_TYPE                                       (false, true,  84, 84, "Stop type", Integer.class),
    HPL_V_1_3_A_INTERCHANGE_VALUE                               (false, true,  85, 85, "Interchange value", Integer.class),
    HPL_V_1_3_A_SPEC_INTERCHANGE_TIME                           (false, true,  86, 87, "Spec. interchange time", Integer.class),
    HPL_V_1_3_A_CLASSIFICATION                                  (false, false, 88, 88, "Classification", Integer.class),



    /*
        Dagkode (DAGKODE.DKO)
     */

    DKO_V_1_1_D_POST_1_DATO                                     (false, false, 1,   6, "Dato", Integer.class),
    DKO_V_1_1_D_POST_1_UKEDAG_FOR_DATO                          (false, false, 7,   7, "Ukedag for dato", Integer.class),

    DKO_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,   1,   3, "Administrasjonskode", Integer.class),
    DKO_V_1_1_D_LOPENR                                          (true, true,   4,   4, "Løpenr", Integer.class),
    DKO_V_1_1_D_DAGKODENR                                       (true, true,   5,   8, "Dagkodenr", Integer.class),

    //The specification says Dagkode is a numerical value, but this makes NO sense as the byte-order is important throughout the value
    DKO_V_1_1_D_DAGKODE                                         (false, true,  9, 400, "Dagkode", String.class),



    DKO_V_1_2_POST_1_DATO                                       (false, true,  1,   6, "Dato", Integer.class),
    DKO_V_1_2_POST_1_UKEDAG_FOR_DATO                            (false, true,  7,   7, "Ukedag for dato", Integer.class),

    DKO_V_1_2_ADMINISTRASJONSKODE                               (true, true,   1,   3, "Administrasjonskode", Integer.class),
    DKO_V_1_2_LOPENR                                            (true, true,   4,   4, "Løpenr", Integer.class),
    DKO_V_1_2_DAGKODENR                                         (true, true,   5,   8, "Dagkodenr", Integer.class),
    DKO_V_1_2_DAGKODE                                           (false, true,  9, 400, "Dagkode", String.class),



    DKO_V_1_3_A_POST_1_DATE                                     (false, true,  1,  6, "Date", Integer.class),
    DKO_V_1_3_A_POST_1_WEEKDAY_FOR_DATE                         (false, true,  7,  7, "Weekday for date", Integer.class),

    DKO_V_1_3_A_ADMINISTRATION_CODE                             (true, true,   1,   3, "Administration code", Integer.class),
    DKO_V_1_3_A_INDEX_NUMBER                                    (true, true,   4,   4, "Index number", Integer.class),
    DKO_V_1_3_A_DAY_CODE_NUMBER                                 (true, true,   5,   8, "Day code number", Integer.class),
    DKO_V_1_3_A_DAY_CODE                                        (false, true,  9, 400, "Day code", String.class);

//
//
//    /*
//        Destinasjon (DESTNAVN.DST)
//     */
//
//    DST_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,   1,  3, "Administrasjonskode"),
//    DST_V_1_1_D_LOPENR                                          (true, true,   4,  4, "Løpenr"),
//    DST_V_1_1_D_DESTINASJONSNUMMER                              (true, true,   5,  8, "Destinasjonsnummer"),
//    DST_V_1_1_D_DESTINASJONSTEKST                               (false, true,  9, 40, "Destinasjonstekst"),
//
//
//
//    DST_V_1_2_ADMINISTRASJONSKODE                               (true, true,   1,  3, "Administrasjonskode"),
//    DST_V_1_2_LOPENR                                            (true, true,   4,  4, "Løpenr"),
//    DST_V_1_2_DESTINASJONSNUMMER                                (true, true,   5,  8, "Destinasjonsnummer"),
//    DST_V_1_2_DESTINASJONSTEKST                                 (false, true,  9, 40, "Destinasjonstekst"),
//
//
//
//    DST_V_1_3_A_ADMINISTRATION_CODE                             (true, true,   1,  3, "Administration code"),
//    DST_V_1_3_A_INDEX_NUMBER                                    (true, true,   4,  4, "Index number"),
//    DST_V_1_3_A_DESTINATION_NUMBER                              (true, true,   5,  8, "Destination number"),
//    DST_V_1_3_A_DESTINATION_TEXT                                (false, true,  9, 40, "Destination text"),
//
//
//
//
//    /*
//        Merknader (MERKNAD.MRK)
//     */
//
//    MRK_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,   1,  3, "Administrasjonskode"),
//    MRK_V_1_1_D_LOPENR                                          (true, true,   4,  4, "Løpenr"),
//    MRK_V_1_1_D_MERKNADSNUMMER                                  (true, true,   5,  7, "Merknadsnummer"),
//    MRK_V_1_1_D_MERKNADSTEKST                                   (false, true,  8, 87, "Merknadstekst"),
//
//
//
//    MRK_V_1_2_ADMINISTRASJONSKODE                               (true, true,   1,  3, "Administrasjonskode"),
//    MRK_V_1_2_LOPENR                                            (true, true,   4,  4, "Løpenr"),
//    MRK_V_1_2_MERKNADSNUMMER                                    (true, true,   5,  7, "Merknadsnummer"),
//    MRK_V_1_2_MERKNADSTEKST                                     (false, true,  8, 87, "Merknadstekst"),
//
//
//
//    MRK_V_1_3_A_ADMINISTRATION_CODE                             (true, true,   1,   3, "Administration code"),
//    MRK_V_1_3_A_INDEX_NUMBER                                    (true, true,   4,   4, "Index number"),
//    MRK_V_1_3_A_REMARK_NUMBER                                   (true, true,   5,   7, "Remark number"),
//    MRK_V_1_3_A_REMARK_TEXT                                     (false, true,  8, 157, "Remark text"),
//
//
//
//    /*
//        Gangvei (GANGVEI.GAV)
//     */
//
//    GAV_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,    1,  3, "Administrasjonskode"),
//    GAV_V_1_1_D_LOPENR                                          (true, true,    4,  4, "Løpenr"),
//    GAV_V_1_1_D_HOLDEPLASSNR_FRA_HOLDEPLASS                     (true, true,    5, 12, "Holdeplassnr"),
//    GAV_V_1_1_D_HOLDEPLASSNR_TIL_HOLDEPLASS                     (true, true,   13, 20, "Holdeplassnr"),
//    GAV_V_1_1_D_GANGTID                                         (false, true,  21, 22, "Gangtid"),
//    GAV_V_1_1_D_GANGVEITEKST                                    (false, false, 23, 42, "Gangveitekst"),
//
//
//
//    GAV_V_1_2_ADMINISTRASJONSKODE                               (true, true,    1,  3, "Administrasjonskode"),
//    GAV_V_1_2_LOPENR                                            (true, true,    4,  4, "Løpenr"),
//    GAV_V_1_2_HOLDEPLASSNR_FRA_HOLDEPLASS                       (true, true,    5, 12, "Holdeplassnr"),
//    GAV_V_1_2_HOLDEPLASSNR_TIL_HOLDEPLASS                       (true, true,   13, 20, "Holdeplassnr"),
//    GAV_V_1_2_GANGTID                                           (false, true,  21, 22, "Gangtid"),
//    GAV_V_1_2_GANGVEITEKST                                      (false, false, 23, 42, "Gangveitekst"),
//
//
//
//    GAV_V_1_3_A_ADMINISTRATION_CODE                             (true, true,    1,  3, "Administration code"),
//    GAV_V_1_3_A_INDEX_NUMBER                                    (true, true,    4,  4, "Index number"),
//    GAV_V_1_3_A_STOP_NUMBER_FROM                                (true, true,    5, 12, "Stop number from"),
//    GAV_V_1_3_A_STOP_NUMBER_TO                                  (true, true,   13, 20, "Stop number to"),
//    GAV_V_1_3_A_WALKWAY_TYPE                                    (true, true,   21, 21, "Walkway type"),           // 1=walk, 2=car, 3=bike
//    GAV_V_1_3_A_TRAVEL_TIME                                     (false, true,  21, 22, "Travel time"),
//    GAV_V_1_3_A_WALKWAY_TEXT                                    (false, false, 23, 42, "Walkway text"),
//
//
//
//    /*
//        Samtrafikk (SAMTFK.SAM)
//     */
//
//    SAM_V_1_1_D_ADMINISTRASJONSKODE_ANKOMST                     (true, true,    1,  3, "Administrasjonskode ankomst"),
//    SAM_V_1_1_D_LOPENR_ANKOMST                                  (true, true,    4,  4, "Løpenr ankomst"),
//    SAM_V_1_1_D_LINJENR_ANKOMST                                 (true, true,    5,  8, "Linjenr ankomst"),
//    SAM_V_1_1_D_TURNR_ANKOMST                                   (true, true,    9, 12, "Turnr ankomst"),
//    SAM_V_1_1_D_TRAFIKKART_ANKOMST                              (true, true,   13, 15, "Trafikkart ankomst"),
//    SAM_V_1_1_D_ADMINISTRASJONSKODE_AVGANG                      (true, true,   16, 18, "Administrasjonskode avgang"),
//    SAM_V_1_1_D_LOPENR_AVGANG                                   (true, true,   19, 19, "Løpenr avgang"),
//    SAM_V_1_1_D_LINJENR_AVGANG                                  (true, true,   20, 23, "Linjenr avgang"),
//    SAM_V_1_1_D_TURNR_AVGANG                                    (true, true,   24, 27, "Turnr avgang"),
//    SAM_V_1_1_D_TRAFIKKART_AVGANG                               (true, true,   28, 30, "Trafikkart avgang"),
//    SAM_V_1_1_D_ANKOMSTTID_ANKOMST                              (false, true,  31, 34, "Ankomsttid tur 1"),
//    SAM_V_1_1_D_ANKOMSTTID_AVGANG                               (false, true,  35, 38, "Avgangstid tur 2"),
//    SAM_V_1_1_D_IKKE_I_BRUK                                     (false, false, 39, 42, "Ikke i bruk"),
//    SAM_V_1_1_D_HOLDEPLASSNR                                    (false, true,  43, 50, "Holdeplassnr"),
//    SAM_V_1_1_D_OVERGANGSKODE                                   (false, true,  51, 51, "Overgangskode"),
//
//
//
//    SAM_V_1_2_ADMINISTRASJONSKODE_ANKOMST                       (true, true,    1,  3, "Administrasjonskode ankomst"),
//    SAM_V_1_2_LOPENR_ANKOMST                                    (true, true,    4,  4, "Løpenr ankomst"),
//    SAM_V_1_2_LINJENR_ANKOMST                                   (true, true,    5,  8, "Linjenr ankomst"),
//    SAM_V_1_2_TURNR_ANKOMST                                     (true, true,    9, 12, "Turnr ankomst"),
//    SAM_V_1_2_TRANSPORTMIDDEL_ANKOMST                           (true, true,   13, 15, "Transportmiddel ankomst"),
//    SAM_V_1_2_ADMINISTRASJONSKODE_AVGANG                        (true, true,   16, 18, "Administrasjonskode avgang"),
//    SAM_V_1_2_LOPENR_AVGANG                                     (true, true,   19, 19, "Løpenr avgang"),
//    SAM_V_1_2_LINJENR_AVGANG                                    (true, true,   20, 23, "Linjenr avgang"),
//    SAM_V_1_2_TURNR_AVGANG                                      (true, true,   24, 27, "Turnr avgang"),
//    SAM_V_1_2_TRANSPORTMIDDEL_AVGANG                            (true, true,   28, 30, "Transportmiddel avgang"),
//    SAM_V_1_2_ANKOMSTTID_ANKOMST                                (false, true,  31, 34, "Ankomsttid tur 1"),
//    SAM_V_1_2_ANKOMSTTID_AVGANG                                 (false, true,  35, 38, "Avgangstid tur 2"),
//    SAM_V_1_2_IKKE_I_BRUK                                       (false, false, 39, 42, "Ikke i bruk"),
//    SAM_V_1_2_HOLDEPLASSNR                                      (false, true,  43, 50, "Holdeplassnr"),
//    SAM_V_1_2_OVERGANGSKODE                                     (false, true,  51, 51, "Overgangskode"),
//
//
//
//    SAM_V_1_3_A_ADMINISTRATION_CODE_ARRIVAL                     (true, true,    1,  3, "Administration code arrival"),
//    SAM_V_1_3_A_INDEX_NUMBER_ARRIVAL                            (true, true,    4,  4, "Index number arrival"),
//    SAM_V_1_3_A_LINE_NUMBER_ARRIVAL                             (true, true,    5,  8, "Line number arrival"),
//    SAM_V_1_3_A_TRIP_NUMBER_ARRIVAL                             (true, true,    9, 12, "Trip number arrival"),
//    SAM_V_1_3_A_TRANSPORT_MODE_ARRIVAL                          (true, true,   13, 15, "Transport mode arrival"),
//    SAM_V_1_3_A_ADMINISTRATION_CODE_DEPARTURE                   (true, true,   16, 18, "Administration code departure"),
//    SAM_V_1_3_A_INDEX_NUMBER_DEPARTURE                          (true, true,   19, 19, "Index number departure"),
//    SAM_V_1_3_A_LINE_NUMBER_DEPARTURE                           (true, true,   20, 23, "Line number departure"),
//    SAM_V_1_3_A_TRIP_NUMBER_DEPARTURE                           (true, true,   24, 27, "Trip number departure"),
//    SAM_V_1_3_A_TRANSPORT_MODE_DEPARTURE                        (true, true,   28, 30, "Transport mode departure"),
//    SAM_V_1_3_A_ARRIVAL_TIME_TRIP_1                             (false, true,  31, 34, "Arrival time trip 1"),
//    SAM_V_1_3_A_DEPARTURE_TIME_TRIP_2                           (false, true,  35, 38, "Departure time trip 2"),
//    SAM_V_1_3_A_NOT_IN_USE                                      (false, false, 39, 42, "Not in use"),
//    SAM_V_1_3_A_STOP_NUMBER                                     (false, true,  43, 50, "Stop number"),
//    SAM_V_1_3_A_TRANSFER_CODE                                   (false, true,  51, 51, "Transfer code"),
//
//
//
//    /*
//        Sone (SONE.SON)
//     */
//
//    SON_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,   1,  3, "Administrasjonskode"),
//    SON_V_1_1_D_LOPENR                                          (true, true,   4,  4, "Løpenr"),
//    SON_V_1_1_D_SONENR                                          (true, true,   5,  9, "Sonenr"),
//    SON_V_1_1_D_SONENAVN                                        (false, true, 10, 39, "Sonenavn"),
//    SON_V_1_1_D_Sone                                            (false, true, 40, 46, "Sone"),                   // Pos. 48-53 i HPL
//
//
//
//    SON_V_1_2_ADMINISTRASJONSKODE                               (true, true,   1,  3, "Administrasjonskode"),
//    SON_V_1_2_LOPENR                                            (true, true,   4,  4, "Løpenr"),
//    SON_V_1_2_SONENR                                            (true, true,   5,  9, "Sonenr"),
//    SON_V_1_2_SONENAVN                                          (false, true, 10, 39, "Sonenavn"),
//    SON_V_1_2_Sone                                              (false, true, 40, 46, "Sone"),                   // Pos. 48-53 i HPL
//
//
//
//    SON_V_1_3_A_ADMINISTRATION_CODE                             (true, true,   1,  3, "Administration code"),
//    SON_V_1_3_A_INDEX_NUMBER                                    (true, true,   4,  4, "Index number"),
//    SON_V_1_3_A_ZONE_NUMBER                                     (true, true,   5,  9, "Zone number"),
//    SON_V_1_3_A_ZONE_NAME                                       (false, true, 10, 39, "Zone name"),
//    SON_V_1_3_A_ZONE                                            (false, true, 40, 46, "Zone"),                   // Pos. 48-53 i HPL
//
//
//
//    /*
//        Linje (LINJE.LIN)
//     */
//
//    LIN_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,   1,  3, "Administrasjonskode"),
//    LIN_V_1_1_D_LOPENR                                          (true, true,   4,  4, "Løpenr"),
//    LIN_V_1_1_D_LINJENR                                         (true, true,   5,  8, "Linjenr"),
//    LIN_V_1_1_D_LINJENAVN                                       (false, true,  9, 38, "Linjenavn"),
//    LIN_V_1_1_D_TAKSTKODE                                       (false, true, 39, 39, "Takstkode"),
//
//
//
//    LIN_V_1_2_ADMINISTRASJONSKODE                               (true, true,   1,  3, "Administrasjonskode"),
//    LIN_V_1_2_LOPENR                                            (true, true,   4,  4, "Løpenr"),
//    LIN_V_1_2_LINJENR                                           (true, true,   5,  8, "Linjenr"),
//    LIN_V_1_2_LINJENAVN                                         (false, true,  9, 38, "Linjenavn"),
//    LIN_V_1_2_TAKSTKODE                                         (false, true, 39, 39, "Takstkode"),
//
//
//
//    LIN_V_1_3_A_ADMINISTRATION_CODE                             (true, true,   1,  3, "Administration code"),
//    LIN_V_1_3_A_INDEX_NUMBER                                    (true, true,   4,  4, "Index number"),
//    LIN_V_1_3_A_LINE_NUMBER                                     (true, true,   5,  8, "Line number"),
//    LIN_V_1_3_A_LINE_NAME                                       (false, true, 9, 38, "Line name"),
//    LIN_V_1_3_A_FARE_CODE                                       (false, true, 39, 39, "Fare code"),
//
//
//    /*
//        Vognløp (VOGNLØP.VLP)
//     */
//
//    VLP_V_1_1_D_ADMINISTRASJONSKODE                             (true, true,    1,  3, "Administrasjonskode"),
//    VLP_V_1_1_D_LOPENUMMER                                      (true, true,    4,  4, "Løpenummer"),
//    VLP_V_1_1_D_VOGNLOPNUMMER                                   (true, true,    5, 10, "Vognløpsnummer"),
//    VLP_V_1_1_D_DAGKODENUMMER                                   (true, true,   11, 14, "Dagkodenummer"),      // Peker til post i DAGKODE
//    VLP_V_1_1_D_SEKVENSNUMMER                                   (true, true,   15, 16, "Sekvensnummer"),      // Turens nr. i vognløpsrekken
//    VLP_V_1_1_D_LINJENUMMER                                     (false, true,  17, 20, "Linjenummer"),        // Post 5-8 i TURIX
//    VLP_V_1_1_D_TURNUMMER                                       (false, true,  21, 24, "Turnummer"),          // Post 9-12 i TURIX
//
//
//
//    VLP_V_1_2_ADMINISTRASJONSKODE                               (true, true,    1,  3, "Administrasjonskode"),
//    VLP_V_1_2_LOPENR                                            (true, true,    4,  4, "Løpenr"),
//    VLP_V_1_2_VOGNLOPSNR                                        (true, true,    5, 10, "Vognløpsnr"),
//    VLP_V_1_2_DAGKODENR                                         (true, true,   11, 14, "Dagkodenr"),          // Peker til post i DAGKODE
//    VLP_V_1_2_SEKVENSNR_TUR                                     (true, true,   15, 16, "Sekvensnr tur"),      // Turens nr. i vognløpsrekken
//    VLP_V_1_2_LINJENR                                           (false, true,  17, 20, "Linjenr"),            // Peker til post i TURIX
//    VLP_V_1_2_TURNR                                             (false, true,  21, 24, "Turnr"),              // Peker til post i TURIX
//    VLP_V_1_2_ID_NR                                             (false, false, 25, 29, "ID-nr"),
//
//
//
//    VLP_V_1_3_A_ADMINISTRATION_CODE                             (true, true,    1,  3, "Administration code"),
//    VLP_V_1_3_A_INDEX_NUMBER                                    (true, true,    4,  4, "Index number"),
//    VLP_V_1_3_A_BLOCK_NUMBER                                    (true, true,    5, 10, "Block number"),
//    VLP_V_1_3_A_DAY_CODE_NUMBER                                 (true, true,   11, 14, "Day code number"),      // Links to Day code
//    VLP_V_1_3_A_SEQUENCE_NUMBER_TRIP                            (true, true,   15, 16, "Sequence number trp"),  // The trip number in the block row
//    VLP_V_1_3_A_LINE_NUMBER                                     (false, true,  17, 20, "Line number"),          // Links to Trip index
//    VLP_V_1_3_A_TRIP_NUMBER                                     (false, true,  21, 24, "Trip number"),          // Linkst to Trip index
//    VLP_V_1_3_A_VEHICLE_ID                                      (false, false, 25, 29, "Vehicle-ID"),
//
//
//
//    /*
//        Tabellversjon (TABVER.TAB)
//     */
//
//    TAB_V_1_2_ADMINISTRASJONSKODE                               (true, true,    1,   3, "Administrasjonskode"),
//    TAB_V_1_2_LOPENR                                            (true, true,    4,   4, "Løpenr"),
//    TAB_V_1_2_LINJENR                                           (true, true,    5,   8, "Linjenr"),
//    TAB_V_1_2_PERIODENR                                         (true, true,    9,  18, "Periodenr"),               // Peker til post i PERIODE
//    TAB_V_1_2_TABELLVERSJONSNAVN                                (false, false, 19,  26, "Tabellversjonsnavn"),      // Intern betegnelse i selskap
//    TAB_V_1_2_TABELLTEKST                                       (false, false, 27,  108, "Tabelltekst"),
//
//
//
//    TAB_V_1_3_A_ADMINISTRATION_CODE                             (true, true,    1,   3, "Administration code"),
//    TAB_V_1_3_A_INDEX_NUMBER                                    (true, true,    4,   4, "Index number"),
//    TAB_V_1_3_A_LINE_NUMBER                                     (true, true,    5,   8, "Line number"),
//    TAB_V_1_3_A_PERIOD_NUMBER                                   (true, true,    9,  18, "Period number"),           // Links to post in Period
//    TAB_V_1_3_A_TABLE_VERSION_NAME                              (false, false, 19,  26, "Table version name"),      // Interlan description in company
//    TAB_V_1_3_A_TABELE_TEXT                                     (false, false, 27, 108, "Table text"),
//
//
//
//    /*
//        Periode (PERIODE.PER)
//     */
//
//    PER_V_1_2_ADMINISTRASJONSKODE                               (true, true,    1,  3, "Administrasjonskode"),
//    PER_V_1_2_LOPENR                                            (true, true,    4,  4, "Løpenr"),
//    PER_V_1_2_PERIODENR                                         (true, true,    5, 14, "Periodenr"),
//    PER_V_1_2_SEKVENSNR_DATOINTERVALL                           (true, true,   15, 15, "Sekvensnr datointervall"),
//    PER_V_1_2_STARTDATO                                         (false, false, 16, 21, "Startdato"),
//    PER_V_1_2_SLUTTDATO                                         (false, false, 22, 27, "Sluttdato"),
//    PER_V_1_2_PERIODETEKST                                      (false, false, 28, 100, "Periodetekst"),
//
//
//
//    PER_V_1_3_A_ADMINISTRATION_CODE                             (true, true,    1,  3, "Administration code"),
//    PER_V_1_3_A_INDEX_NUMBER                                    (true, true,    4,  4, "Index number"),
//    PER_V_1_3_A_PERIOD_NUMBER                                   (true, true,    5, 14, "Period number"),
//    PER_V_1_3_A_SEQUENCE_NUMBER_DATE_INTERVAL                   (true, true,   15, 15, "Sequence number date interval"),
//    PER_V_1_3_A_START_DATE                                      (false, true,  16, 21, "Start date"),
//    PER_V_1_3_A_STOP_DATE                                       (false, true,  22, 27, "Stop date"),
//    PER_V_1_3_A_PERIOD_TEXT                                     (false, true,  28, 100, "Period text"),
//
//
//
//    /*
//        Rutepunkt (RUTEPKT.RUT)
//     */
//
//    RUT_V_1_2_ADMINISTRASJONSKODE                               (true, true,    1,  3, "Administrasjonskode"),
//    RUT_V_1_2_LOPENR                                            (true, true,    4,  4, "Løpenr"),
//    RUT_V_1_2_LINJENR                                           (true, true,    5,  8, "Linjenr"),
//    RUT_V_1_2_PERIODENR                                         (true, true,    9, 18, "Periodenr"),
//    RUT_V_1_2_SEKVENSNR_RUTEPKT                                 (true, true,   19, 21, "Sekvensnr rutepkt"),
//    RUT_V_1_2_RETNING                                           (false, true,  22, 22, "Retning"),                // Verdi 1, 2 eller 3
//    RUT_V_1_2_HOLDEPLASSNR                                      (false, true,  23, 30, "Holdeplassnr"),
//
//
//
//    RUT_V_1_3_A_ADMINISTRATION_CODE                             (true, true,    1,  3, "Administration code"),
//    RUT_V_1_3_A_INDEX_NUMBER                                    (true, true,    4,  4, "Index number"),
//    RUT_V_1_3_A_LINE_NUMBER                                     (true, true,    5,  8, "Line number"),
//    RUT_V_1_3_A_PERIOD_NUMBER                                   (true, true,    9, 18, "Period number"),
//    RUT_V_1_3_A_SEQUENCE_NUMBER                                 (true, true,   19, 21, "Sequence number main stop"),
//    RUT_V_1_3_A_DIRECTION                                       (false, true,  22, 22, "Direction"),
//    RUT_V_1_3_A_STOP_NUMBER                                     (false, true,  23, 30, "Stop number"),
//    RUT_V_1_3_A_STOP_POINT_NUMBER                               (false, true,  31, 32, "Stop point number");



    private boolean isKey;
    private boolean isMandatory;
    private int begin;
    private int end;
    private String fieldName;
    private Class cls;

    private Field(boolean isKey, boolean isMandatory, int begin, int end, String fieldName, Class cls) {
        this.isKey = isKey;
        this.isMandatory = isMandatory;
        this.begin = begin - 1;
        this.end = end;
        this.fieldName = fieldName;
        this.cls = cls;
    }


    public boolean isKey() {
        return isKey;
    }


    public boolean isMandatory() {
        return isMandatory;
    }


    public int getBegin() {
        return begin;
    }


    public int getEnd() {
        return end;
    }


    public String getFieldName() {
        return fieldName;
    }

    public Class getCls() {
        return cls;
    }



    public static Field[] getFieldsForFileTypeAndVersion(FileType fileType, Version version) {

        List<Field> fields = new ArrayList<>();

        // Lets avoid hundres of lines of type/version-specific array generation, and take the short cut by using reflection! :)
        // And everyone was happy, for all the remainder of their days \o/
        for(java.lang.reflect.Field field : Field.class.getDeclaredFields()) {
            if(field.getName().startsWith(fileType.name() + "_" + version.name())) {
                fields.add(Field.valueOf(field.getName()));
            }
        }
        return fields.toArray(new Field[fields.size()]);
    }
}
