package RegToppParser.models.regtopp;

import RegToppParser.control.FileTaskDelegator;
import RegToppParser.interfaces.RegToppFileParser;
import RegToppParser.models.Field;
import RegToppParser.models.FileType;
import RegToppParser.models.Version;
import RegToppParser.util.ParseUtil;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Map;
import java.util.Scanner;

/**
 * Created on 15.11.14.
 * @author Espen Jacobsson
 */
public class DKO extends RegToppObject implements RegToppFileParser {


    private String line = "";
    private int dato = 0; // Merknad
    private int ukedagForDato = 0;


    public DKO(Version version) {
        super(version, Field.getFieldsForFileTypeAndVersion(FileType.DKO, version));
        FileTaskDelegator.getInstance().registerFileParser(this, FileType.DKO);
    }


    public String getLine() {
        return line;
    }


    public int getDate() {
        return dato;
    }


    public int getWeekdayForDate() {
        return ukedagForDato;
    }


    public boolean parse(File file) {

        try(Scanner scanner = new Scanner(file)) {

            // Parse the first line of the DKO file
            getFirstLineValues(scanner);

            // Parse the rest of the file
            while(scanner.hasNext()) {
                String line = scanner.nextLine();
                super.parse(line);
            }
        }
        catch (FileNotFoundException e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }


    private void getFirstLineValues(Scanner scanner) {

        String firstLine = scanner.nextLine();
        Map<Field, Object> firstLineValues;

        switch(this.getVersion()) {
            case V_1_1_D: {
                firstLineValues     = ParseUtil.parseLine(firstLine, Field.DKO_V_1_1_D_POST_1_DATO, Field.DKO_V_1_1_D_POST_1_UKEDAG_FOR_DATO);
                this.dato           = (int) firstLineValues.get(Field.DKO_V_1_1_D_POST_1_DATO);
                this.ukedagForDato  = (int) firstLineValues.get(Field.DKO_V_1_1_D_POST_1_UKEDAG_FOR_DATO);
                break;
            }
            case V_1_2: {
                firstLineValues     = ParseUtil.parseLine(firstLine, Field.DKO_V_1_2_POST_1_DATO, Field.DKO_V_1_2_POST_1_UKEDAG_FOR_DATO);
                this.dato           = (int) firstLineValues.get(Field.DKO_V_1_2_POST_1_DATO);
                this.ukedagForDato  = (int) firstLineValues.get(Field.DKO_V_1_2_POST_1_UKEDAG_FOR_DATO);
                break;
            }
            case V_1_3_A: {
                firstLineValues     = ParseUtil.parseLine(firstLine, Field.DKO_V_1_3_A_POST_1_DATE, Field.DKO_V_1_3_A_POST_1_WEEKDAY_FOR_DATE);
                this.dato           = (int) firstLineValues.get(Field.DKO_V_1_3_A_POST_1_DATE);
                this.ukedagForDato  = (int) firstLineValues.get(Field.DKO_V_1_3_A_POST_1_WEEKDAY_FOR_DATE);
                break;
            }
        }
    }
}
