package RegToppParser.models.regtopp;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.*;

import RegToppParser.models.Field;
import RegToppParser.models.FieldValuePair;
import RegToppParser.models.Post;
import RegToppParser.models.Version;
import RegToppParser.util.ParseUtil;

/**
 * Created on 15.11.14.
 * @author Espen Jacobsson
 */
public class RegToppObject {


    Version version;
    Field[] fields = null;
    Map<Map<Field, Object>, Post> properFieldToPostMapping = new HashMap<>();


    public RegToppObject(Version version, Field... fields) {
        this.version = version;
        this.fields = fields;
    }


    public Version getVersion() {
        return version;
    }
    
    
    protected void parse(String line) {
        Map<Field, Object> fieldValues = ParseUtil.parseLine(line, fields);
        Post post = new Post(line);
        post.addFields(fieldValues);

        mapFieldsToPosts(post, fieldValues);
    }


    protected boolean parseFile(File file) {

//        try(Scanner scanner = new Scanner(file, Charset.forName("iso-8859-1").name())) {
        try {
            List<String> lines = Files.readAllLines(file.toPath(), Charset.forName("iso-8859-1"));

            for(String line : lines) {
                if(line.trim().isEmpty()) continue;
                parse(line);
            }
        }
        catch (IOException e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    private void mapFieldsToPosts(Post post, Map<Field, Object> fieldValues) {

        Map<Field, Object> primaryKeyMap = new HashMap<>();

        for(Field field : fieldValues.keySet()) {
            if(!field.isKey()) continue;

            primaryKeyMap.put(field, fieldValues.get(field));
        }

        properFieldToPostMapping.put(primaryKeyMap, post);

    }

    public Post getPost(List<FieldValuePair> fieldValuePairs) {

        Map<Field, Object> primaryKeyMap = new HashMap<>();

        for(FieldValuePair pair : fieldValuePairs) {
            primaryKeyMap.put(pair.getField(), pair.getValue());
        }

        if(!properFieldToPostMapping.containsKey(primaryKeyMap))
            return null;

        return properFieldToPostMapping.get(primaryKeyMap);
    }


    public List<Post> getPosts() {
        return new ArrayList<>(properFieldToPostMapping.values());
    }
}
