package RegToppParser.util;

import RegToppParser.models.Field;

import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

/**
 * Created on 15.11.14.
 * @author Espen Jacobsson
 */
public class ParseUtil {
    

    /**
     * Parses multiple fields and returns the names and values as a Map.
     * The line must be specific to the fields info given.
     * @param line String of data
     * @param fields Information about the fields to be parsed
     * @return
     */
    public static Map<Field, Object> parseLine(String line, Field... fields) {
        
        Map<Field, Object> fieldValues = new HashMap<>();

        for(Field field : fields) {
            
            if(!fieldValues.containsKey(field)) {
                fieldValues.put(field, parseLine(line, field));
            }
        }
        
        return fieldValues;
    }
    
    
    /**
     * Parses a line of text from the given Field.
     * The line must be specific to the field info given.
     * @param line String of data
     * @param field Information about the field to be parsed
     * @return Returns the value of a field on a line
     */
    public static Object parseLine(String line, Field field) {
        try {

            int begin = field.getBegin();
            int end = field.getEnd();

            if(begin < 0)
                begin = 0;

            if(begin > line.length())
                begin = line.length() - 1;

            if(end < 0)
                end = 0;

            if(end > line.length())
                end = line.length() -1;

            if(end < begin)
                end = begin;

            String value = line.substring(begin, end);

            if(field.getCls() == Integer.class) {
                return Integer.parseInt(value.trim());
            }
            else if(field.getCls() == String.class) {
                return String.valueOf(value.trim());
            }
        }
        catch(StringIndexOutOfBoundsException oobException) {
            oobException.printStackTrace();
        }
        return "";
    }
}
