package espenja;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;

import espenja.exceptions.InvalidHPLException;
import espenja.model.HPL;


public class StatCoordFileParser {

    public static String readFileHeader() throws UnsupportedEncodingException, FileNotFoundException, IOException {

        try(BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(Settings.statCoordInputFile.toFile()), Charset.forName(Settings.INENCODING).toString()))) {
            String line = reader.readLine();

            Matcher encodingMatcher = Settings.encodingPattern.matcher(line);

            if (encodingMatcher.find()) {
                System.out.println("Encoding is: " + encodingMatcher.group(1));

                if (encodingMatcher.group(1).equalsIgnoreCase(Settings.INENCODING)) {
                    return line;
                }
                else {
                    throw new UnsupportedEncodingException("UNEXPECTED ENCODING is "+encodingMatcher.group(1));
                }
            }
            
            System.out.println("No header found, using utf-8 as encoding");
            return "/* -*- Mode:Prolog; coding:utf-8; -*- */";
        }
    }

    public static Map<String, HPL> readContents() throws UnsupportedEncodingException, FileNotFoundException, IOException, InvalidHPLException {

        try(BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(Settings.statCoordInputFile.toFile()), Charset.forName(Settings.INENCODING).toString()))) {
            
            //Skip first line, we have already read the header.
            reader.readLine();
            //Add a line to separate the header section
            StatCoordFileWriter.getInstance().addLine("");
            
            Map<String, HPL> statCoords = new HashMap<String, HPL>();
            String line;

            while((line = reader.readLine()) != null) {

                Matcher statCoordMatcher = Settings.statCoordPattern.matcher(line);

                if(statCoordMatcher.find()) {
                    String regtoppID = statCoordMatcher.group(1);
                    String name = statCoordMatcher.group(2);
                    String lat = statCoordMatcher.group(3);
                    String lng = statCoordMatcher.group(4);

                    statCoords.put(regtoppID, new HPL(regtoppID, name, null, null, null, lat, lng, null));
                }
                else {
                    StatCoordFileWriter.getInstance().addLine(line);
                }
            }
            
            //Add a line to separate commented and edited fields
            StatCoordFileWriter.getInstance().addHeader("");
            
            return statCoords;
        }
    }
}
