package espenja;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import espenja.converter.CSVConverter;
import espenja.exceptions.InvalidHPLException;
import espenja.model.HPL;



public class StatCoordUpdater {
    
    public static void main(String[] args) throws UnsupportedEncodingException, FileNotFoundException, InvalidHPLException, IOException {
        
      Map<String, HPL> temp = CSVConverter.convertStopsToHPL();
      CSVConverter.writeJson(temp);
        
        new StatCoordUpdater();
    }

    
    private Map<String, HPL> newEntries = new HashMap<>();
    private Map<String, HPL> oldEntries = new HashMap<>();
    private Map<String, HPL> sortedEntries = new TreeMap<>();
    
    
    public StatCoordUpdater() {
        
        try {

            String statCoordHeader = StatCoordFileParser.readFileHeader();
            StatCoordFileWriter.getInstance().addHeader(statCoordHeader);
            
            oldEntries = StatCoordFileParser.readContents();
            newEntries = readNewEntries();
            
            compareAndUpdateCoordStats();
            writeCoordStats();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        catch (InvalidHPLException e) {
            e.printStackTrace();
        }
    }
    
    
    private Map<String, HPL> readNewEntries() throws IOException {
        
        List<String> lines = Files.readAllLines(Settings.jsonOutputFile, Charset.forName(Settings.INENCODING));
        StringBuilder builder = new StringBuilder();
        
        for(String line : lines) {
            builder.append(line);
        }
        
        Gson gson = new Gson();
        return gson.fromJson(builder.toString(), new TypeToken<Map<String, HPL>>() { }.getType());
    }
    
    
    private void compareAndUpdateCoordStats() {
        
        System.out.println("Updating " + newEntries.size() + " entries.");
        
        int initialSize = oldEntries.size();
        
        for(String regtoppID : newEntries.keySet()) {
            
            if(!regtoppID.startsWith("'")) {
                regtoppID = "'" + regtoppID + "'";
            }
            
            if(oldEntries.containsKey(regtoppID)) {
                System.out.println(oldEntries.get(regtoppID));
                oldEntries.remove(regtoppID);
            }
        }
        
        System.out.println(initialSize - oldEntries.size() + " entries matched ID and are scheduled for updating!");
        
        sortedEntries.putAll(newEntries);
        sortedEntries.putAll(oldEntries);
    }
    
    
    private void writeCoordStats() {
        
        for(String regtoppID : sortedEntries.keySet()) {
            HPL hpl = sortedEntries.get(regtoppID);
            StatCoordFileWriter.getInstance().addLine(hpl.asPrologStatCoordLine());
        }
        
        StatCoordFileWriter.getInstance().writeFile();
    }
}
