package espenja.converter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import espenja.exceptions.InvalidHPLException;
import espenja.model.HPL;


public class CSVConverter {

    private static Path osloStopsCSVFile = Paths.get("data/stops2.csv");
    private static Path osloHplFile = Paths.get("data/r0320.hpl");
    private static Path jsonOutputFile = Paths.get("data/oslo.json");
    

    public static Map<String, HPL> convertStopsToHPL() throws InvalidHPLException, UnsupportedEncodingException, FileNotFoundException, IOException {
        try(BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(osloStopsCSVFile.toFile()), Charset.forName("UTF-16").toString()))) {

            Map<String, HPL> stops = new HashMap<>();
            String line = null;
            
            stops.putAll(convertHplFileToHPL());

            while((line = reader.readLine()) != null) {

                String[] fragments = line.split(";");

                String stopId               = fragments[0];
                String name                 = fragments[1];
                String shortName            = fragments[2];
                String municipacity         = fragments[3];
                String zone                 = fragments[4];
                String x                    = fragments[5];
                String y                    = fragments[6];
                String realTimeDatabaseID   = fragments[7];

                stops.put(stopId, new HPL(stopId, name, shortName, municipacity, zone, x, y, realTimeDatabaseID));
            }

            return stops;
        }
    }
    
    private static Map<String, HPL> convertHplFileToHPL() throws FileNotFoundException, IOException, InvalidHPLException {
        try(BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(osloHplFile.toFile()), Charset.forName("iso-8859-1")))) {
            
           Pattern xCoordPattern = Pattern.compile("^0+(\\d{6})");
           Pattern yCoordPattern = Pattern.compile("^0+(\\d{7})");
            
           Map<String, HPL> stops = new HashMap<>();
           String line = null;
           
           while((line = reader.readLine()) != null) {
               
               String stopId = line.substring(4, 12).replaceAll("^0+", "");
               String name = line.substring(12, 42).trim();
               String shortName = line.substring(42, 47).trim();
               String zone = line.substring(47, 53).trim();
               
               String x = line.substring(53,63);
               String y = line.substring(63,73);
               
               try {
                   
                   Matcher xCoordMatcher = xCoordPattern.matcher(line.substring(53,63));
                   Matcher yCoordMatcher = yCoordPattern.matcher(line.substring(63,73));
                   
                   if(xCoordMatcher.find()) {
                       x = xCoordMatcher.group(1) + "0";
                   }
                   
                   if(yCoordMatcher.find()) {
                       y = yCoordMatcher.group(1) + "0";
                   }
               } catch(Exception e) {
                   x = "0";
                   y = "0";
               }
               
               if(x.replaceAll("0", "").length() == 1 || y.replaceAll("0", "").length() == 1) {
                   x = "0";
                   y = "0";
               }
               
               stops.put(stopId, new HPL(stopId, name, shortName, "", zone, x, y, ""));
               
           }
           
           return stops;
        }
    }


    public static void writeJson(Map<String, HPL> stops) {

        try(BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(jsonOutputFile.toFile()), Charset.forName("UTF-8").toString()))) {
            Gson gson = new GsonBuilder().setPrettyPrinting().create();
            String json = gson.toJson(stops).toString();

            writer.write(json);
        }
        catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }
}