package espenja.model;

import com.ibm.util.CoordinateConversion;

import espenja.exceptions.InvalidHPLException;


public class HPL {
    
    private String regtoppID;
    private String name;
    private String shortName;
    private String municipality;
    private String zone;
    private String x;
    private String y;
    private String lat;
    private String lng;
    private String realTimeDatabaseID;

    
    public HPL(String regtoppID, String name, String shortName, String municipality, String zone, String x, String y, String realTimeDatabaseID) throws InvalidHPLException {
        this.regtoppID = regtoppID;
        this.name = name;
        this.shortName = shortName;
        this.municipality = municipality;
        this.zone = zone;
        this.x = x;
        this.y = y;
        this.realTimeDatabaseID = realTimeDatabaseID;
        
        convertCoordinates();
        if(!validate()) {
            throw new InvalidHPLException();
        }
    }
    
    
    private void convertCoordinates() {
        try {
            CoordinateConversion coordinateConversion = new CoordinateConversion();
            
            double latCheck = Double.parseDouble(x);
            double lngCheck = Double.parseDouble(y);
            
            // If one or both of the coordinates are 0, the conversion will return weird results, set both to 0
            if(latCheck == 0.0 || lngCheck == 0.0) {
                lat = "0";
                lng = "0";
                return;
            }
            
            String utm = String.format("%s %s %s", "32 V", x, y);
            
            double[] latLng = coordinateConversion.utm2LatLon(utm);
            
            lat = String.valueOf(latLng[0]);
            lng = String.valueOf(latLng[1]);
            
        } catch(NumberFormatException nfE) {
            nfE.printStackTrace();
        }
    }
    
    private boolean validate() {
        
        if(regtoppID == null || regtoppID.trim().isEmpty()) return false;
        if(name == null || name.trim().isEmpty()) return false;
        
        try {
            double latitude = Double.parseDouble(lat);
            double longitude = Double.parseDouble(lng);
            
            CoordinateConversion coordinateConversion = new CoordinateConversion();
            coordinateConversion.validate(latitude, longitude);
            
        } catch(NumberFormatException nfE) {
            nfE.printStackTrace();
            return false;
        } catch(IllegalArgumentException iaE) {
            iaE.printStackTrace();
            return false;
        }
        
        return true;
    }
    

    public String getRegtoppID() {
        return regtoppID;
    }

    
    public void setRegtoppID(String regtoppID) {
        this.regtoppID = regtoppID;
    }
    
    
    public String getName() {
        return name;
    }
    
    
    public void setName(String name) {
        this.name = name;
    }
    
    
    public String getShortName() {
        return shortName;
    }
    
    
    public void setShortName(String shortName) {
        this.shortName = shortName;
    }
    
    
    public String getMunicipality() {
        return municipality;
    }
    
    
    public void setMunicipality(String municipality) {
        this.municipality = municipality;
    }
    
    
    public String getZone() {
        return zone;
    }
    
    
    public void setZone(String zone) {
        this.zone = zone;
    }
    
    
    public String getX() {
        return x;
    }
    
    
    public void setX(String x) {
        this.x = x;
    }
    
    
    public String getY() {
        return y;
    }
    
    
    public void setY(String y) {
        this.y = y;
    }
    
    
    public String getLag() {
        return lat;
    }
    
    
    public void setLat(String lat) {
        this.lat = lat;
    }
    
    
    public String getLng() {
        return lng;
    }
    
    
    public void setLng(String lng) {
        this.lng = lng;
    }
    
    
    public String getRealTimeDatabaseID() {
        return realTimeDatabaseID;
    }
    
    
    public void setRealTimeDatabaseID(String realTimeDatabaseID) {
        this.realTimeDatabaseID = realTimeDatabaseID;
    }


    public String asPrologStatCoordLine() {
        if(!validate()) new InvalidHPLException();
        
        return String.format("statcoord('%s', '%s', %s, %s).", regtoppID, name, lat, lng);
    }
    
    
    @Override
    public String toString() {
        return asPrologStatCoordLine();
    }
}
