
package regtopToBusTUC;

import java.io.*;
import java.util.*;

/** Class for the prolog predicates regbus and route.<BR>
	@author Tore Bruland
	@author satre
	@since 111201 */
public class RegbusHolder {

	/** @param bustucOutputRootFolder Where to write the regxxx.pl files (for bustuc/db/tables) */
	public RegbusHolder( String bustucOutputRootFolder ) {
		directory = bustucOutputRootFolder;
		regbus = new ArrayList<String>();
		route = new ArrayList<String>();
	}

	/** instance is used to hold several filesets<br>
		method is called before each new fileset is read  */
	public void clear() {
		directory="";
		regbus.clear();
		route.clear();
	}

	/** collects data from the tix file */
	public void setTix(String iLine, String iTour){
		//  String line=Util.trimZero(iLine); // %% TA-110209
		//  String tour=Util.trimZero(iTour);

		String line=iLine.trim(); // %% TA-110209
		String tour=iTour.trim();

		if ( !regbus.contains(line)) {
			regbus.add(line);
		}
		
		/* HER */
		
		String key;
		
		if(Settings.useEncasingCharacter) {
		    //Remove all leading zeros and replace with one zero
		    //This because coordinates file disregards the zeros
		    String lineCleaned = line.replaceAll("^0+", "");
		    if(lineCleaned.isEmpty()) lineCleaned = "0";
		    
		    lineCleaned = Settings.encasingCharacter + lineCleaned + Settings.encasingCharacter;
		    
		    key = String.format("route(bus_%s_%s,%s,%s).", line, tour, lineCleaned, lineCleaned);
		}
		else {
		    key = String.format("route(bus_%s_%s,%s,%s).", line, tour, line, line);
		}
		if ( !route.contains(key) ) {
			route.add(key);
		}//if not existing key
	}//setTix


//	public void OLDsetTix(String iLine, String iTour) {
//		String line=iLine;  //  Util.trimZero(iLine); // %% TA-110209
//		String tour=iTour;  //  Util.trimZero(iTour);    %% TA-110209
//		String key = "bus_"+line+"_"+tour;
//
//		if ( !regbus.contains(line)) {
//			regbus.add(line);
//		}
//		//if ( !route.contains(key) ) {
//		/* " HER */
//		route.add("route("+key+",'"+line+"','"+line+"').");
//		//}
//	}//OLDsetTix


	/** write the records of the regpas and route predicates to the proper directory and filename regpas.pl */
	public void writePredicates(){
		try {
			BufferedWriter outFile = new BufferedWriter( new FileWriter( directory+File.separator+"regbus.pl" ) );
			
			if(ConvertRegTop.OUTPUT_ENCODING.equals("utf-8"))
				outFile.write("/* -*- Mode:Prolog; coding:utf-8; -*- */\n");
			else
				outFile.write("/* -*- Mode:Prolog; coding:iso-8859-1; -*- */\n");
			
			for(String lineNr : regbus) {
			    /* " HER */
			    if(Settings.useEncasingCharacter) {
			        
			        //Replace many zeroes at the start of the lineNr with zero zeros HE HE HE
			        //This because the coordinate files disregard the leading zeros
			        lineNr = lineNr.replaceAll("^0+", "");
			        //If the lineNr is then empty, the lineNr is 0.
			        if(lineNr.equals("")) lineNr = "0";
			        
			        outFile.write(String.format("regbus(%s%s%s).\n", Settings.encasingCharacter, lineNr, Settings.encasingCharacter));
			    }
			    else {
			        outFile.write(String.format("regbus(%s)\n)", lineNr));
			    }
			}
			outFile.write("\n\n");
			for ( Iterator< String > iter=route.iterator(); iter.hasNext();  ){ //iter.next
				outFile.write( iter.next()+"\n" );
			}				
			outFile.close();
		} catch (IOException e ) {
			System.out.println("********** IOException "+e);
			e.printStackTrace();
		}		
	}

	/**
		holds the records of the regpas predicate
	 */
	public ArrayList<String> regbus;

	/** holds the records of the route predicate */	
	public ArrayList<String> route;

	/**	name of the directory */		
	public String directory;

}//class RegbusHolder
