package regtopToBusTUC;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
//import java.nio.charset.Charset;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class TopRegUpdate {

	public static final String OUTPUT_ENCODING = ConvertRegTop.OUTPUT_ENCODING; //UTF-8 !
	//public static final String OUTPUT_ENCODING = "utf-8";
	//private static final String INPUT_ENCODING = "iso-8859-1";

	/** Store all the route_period lines, in UTF-8 encoding! */
	public static List< RoutePeriod > routePeriod;

	public static void updateTopreg( String comment, String admCode, Long timeFrom, Long timeTo, String path ){
		routePeriod = new ArrayList< RoutePeriod >();

		try { //Fix encoding
			comment = URLDecoder.decode( comment, OUTPUT_ENCODING );
		} catch (UnsupportedEncodingException e) { e.printStackTrace(); }

		//			seconds since 1970
		Date fromDate = new Date( timeFrom * 1000 );
		Date toDate = new Date( timeTo * 1000 );

		SimpleDateFormat folderDate = new SimpleDateFormat("yyMMdd");
		String folderName = admCode.toLowerCase()+"_"+folderDate.format(fromDate);

		SimpleDateFormat period = new SimpleDateFormat("yyyy,MM,dd");
		String to = period.format(toDate);
		String from = period.format(fromDate);

		//route_period( tt, r1612_121001, date(2012,10,01),    date(2012,12,23) ). 
		String newRoute = "route_period(   tt, "+folderName+", date("+from+"),   date("+to+") ).	%% "+comment;
		RoutePeriod newPeriod = new RoutePeriod( newRoute );
		routePeriod.add( newPeriod );

		//				routes.pl
		FileInputStream input;
		DataInputStream data;
		BufferedReader reader;
		try {
			input = new FileInputStream( path );
			data = new DataInputStream( input);
			reader = new BufferedReader( new InputStreamReader( data, OUTPUT_ENCODING ) );
			String strLine;
			while ( (strLine = reader.readLine() ) != null ){
				if( strLine.startsWith("%%route_period") || strLine.startsWith("route_period") ){
					RoutePeriod p = new RoutePeriod( strLine );
					routePeriod.add( p );
				}
			}
			reader.close();
			data.close();
			input.close();
		}catch (FileNotFoundException e) {
			System.out.println("File not found: "+e);
			e.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}
		Collections.sort(routePeriod);
		writeFile(path);
	}//updateTopreg


	private static void writeFile( String path ){
		try {
			BufferedWriter outFile = new BufferedWriter( new OutputStreamWriter( new FileOutputStream(path), OUTPUT_ENCODING ) );

			if( ConvertRegTop.OUTPUT_ENCODING.toLowerCase().equals( "utf-8" ) ){
				outFile.write("/* -*- Mode:Prolog; coding:utf-8; -*- */\n");
			}else{
				outFile.write("/* -*- Mode:Prolog; coding:iso-8859-1; -*- */\n");
			}
			//outFile.write("%% Generated by regtopToBusTUC.TopRegUpdate.java on "+new Timestamp(new Date().getTime())+"\n\n");
			outFile.write("%% Generated by "+TopRegUpdate.class+" on "+new Timestamp( new Date().getTime() )+"\n\n");
			outFile.write("% Xmas routes before winter routes\n"+
					"% Easter routes before winter routes\n\n"+
					"%% Route module 'nil' in route_period means that the route module is not implemented yet\n"+
					"%% Important to have dummy route modules to give message of no routes for this date\n\n");
			outFile.write("%%%% ACTUAL  ROUTES              START-DATE           END-DATE (sorted!)\n\n");

			for ( Iterator< RoutePeriod > iter = routePeriod.iterator(); iter.hasNext(); ){
				outFile.write(iter.next().getRoute()+"\n");
			}
			outFile.close();
		} catch (IOException e ) {
			System.out.println("********** IOException "+e);
			e.printStackTrace();
		}
	}//writeFile

	public static void main( String[] args ){
		if( args.length != 5 ){
			String usage = "regtopToBusTUC.TopRegUpdate [comment] [admCode] [fromDate] [toDate] [filePath] " +
					"Append a route period predicat at the end of topreg.pl in [filepath]";
			System.err.println( usage );
			System.exit( 1 );
		}else{
			try{
				TopRegUpdate.updateTopreg( args[0], args[1], Long.parseLong( args[2] ), Long.parseLong( args[3] ), args[4] );
				System.out.println("Conversion completed!");
				//			}catch(FileNotFoundException e){
				//				System.err.println("Failed to update -> File unavailable: "+e);
			}catch(Exception e){
				System.err.println("Failed to update: "+e);
			}
		}
	}//main

}//class TopRegUpdate
